@@ -19,14 +19,20 @@ from concurrent.futures import ThreadPoolExecutor
 from contextlib import contextmanager
 from typing import Callable, Optional, Union
 
+import numpy as np
 import pandas as pd
+import psutil
+import torch
 
 from ludwig.data.cache.manager import CacheManager
 from ludwig.data.dataframe.pandas import PANDAS
 from ludwig.data.dataset.base import DatasetManager
 from ludwig.data.dataset.pandas import PandasDatasetManager
-from ludwig.models.ecd import ECD
-from ludwig.utils.fs_utils import get_bytes_obj_if_path
+from ludwig.models.base import BaseModel
+from ludwig.schema.trainer import ECDTrainerConfig, GBMTrainerConfig
+from ludwig.utils.fs_utils import get_bytes_obj_from_path
+from ludwig.utils.misc_utils import get_from_registry
+from ludwig.utils.system_utils import Resources
 from ludwig.utils.torch_utils import initialize_pytorch
 from ludwig.utils.types import Series
 
@@ -59,7 +65,7 @@ class Backend(ABC):
 
     @contextmanager
     @abstractmethod
-    def create_trainer(self, **kwargs):
+    def create_trainer(self, **kwargs) -> "BaseTrainer":  # noqa: F821
         raise NotImplementedError()
 
     @abstractmethod
@@ -97,6 +103,10 @@ class Backend(ABC):
     def num_nodes(self) -> int:
         raise NotImplementedError()
 
+    @abstractmethod
+    def get_available_resources(self) -> Resources:
+        raise NotImplementedError()
+
 
 class LocalPreprocessingMixin:
     @property
@@ -110,27 +120,43 @@ class LocalPreprocessingMixin:
     def check_lazy_load_supported(self, feature):
         pass
 
-    def read_binary_files(self, column: pd.Series, map_fn: Optional[Callable] = None) -> pd.Series:
-        df = column.to_frame(name=column.name)
+    def read_binary_files(
+        self, column: pd.Series, map_fn: Optional[Callable] = None, file_size: Optional[int] = None
+    ) -> pd.Series:
+        column = column.fillna(np.nan).replace([np.nan], [None])  # normalize NaNs to None
 
+        sample_fname = column.head(1).values[0]
         with ThreadPoolExecutor() as executor:  # number of threads is inferred
-            result = executor.map(lambda idx_and_row: get_bytes_obj_if_path(idx_and_row[1][column.name]), df.iterrows())
+            if isinstance(sample_fname, str):
+                result = executor.map(
+                    lambda path: get_bytes_obj_from_path(path) if path is not None else path, column.values
+                )
+            else:
+                # If the sample path is not a string, assume the paths has already been read in
+                result = column.values
+
             if map_fn is not None:
                 result = executor.map(map_fn, result)
 
-        return pd.Series(result, index=df.index, name=column.name)
+        return pd.Series(result, index=column.index, name=column.name)
 
 
 class LocalTrainingMixin:
     def initialize_pytorch(self, *args, **kwargs):
         initialize_pytorch(*args, **kwargs)
 
-    def create_trainer(self, **kwargs):
-        from ludwig.models.trainer import Trainer
+    def create_trainer(
+        self, config: Union[ECDTrainerConfig, GBMTrainerConfig], model: BaseModel, **kwargs
+    ) -> "BaseTrainer":  # noqa: F821
+        from ludwig.trainers.registry import trainers_registry
 
-        return Trainer(**kwargs)
+        trainers_for_model = get_from_registry(model.type(), trainers_registry)
 
-    def create_predictor(self, model: ECD, **kwargs):
+        trainer_cls = get_from_registry(config.type, trainers_for_model)
+
+        return trainer_cls(config=config, model=model, **kwargs)
+
+    def create_predictor(self, model: BaseModel, **kwargs):
         from ludwig.models.predictor import Predictor
 
         return Predictor(model, **kwargs)
@@ -157,6 +183,8 @@ class RemoteTrainingMixin:
 
 
 class LocalBackend(LocalPreprocessingMixin, LocalTrainingMixin, Backend):
+    BACKEND_TYPE = "local"
+
     def __init__(self, **kwargs):
         super().__init__(dataset_manager=PandasDatasetManager(self), **kwargs)
 
@@ -166,3 +194,6 @@ class LocalBackend(LocalPreprocessingMixin, LocalTrainingMixin, Backend):
     @property
     def num_nodes(self) -> int:
         return 1
+
+    def get_available_resources(self) -> Resources:
+        return Resources(cpus=psutil.cpu_count(), gpus=torch.cuda.device_count())
