@@ -17,6 +17,7 @@ import argparse
 import sys
 
 import ludwig.contrib
+from ludwig.utils.print_utils import get_logo
 
 
 class CLI:
@@ -28,7 +29,8 @@ class CLI:
     def __init__(self):
         parser = argparse.ArgumentParser(
             description="ludwig cli runner",
-            usage="""ludwig <command> [<args>]
+            usage=f"""\n{get_logo("ludwig cli", ludwig.__version__)}
+ludwig <command> [<args>]
 
 Available sub-commands:
    train                 Trains a model
@@ -36,6 +38,7 @@ Available sub-commands:
    evaluate              Evaluate a pretrained model's performance
    experiment            Runs a full experiment training a model and evaluating it
    hyperopt              Perform hyperparameter optimization
+   benchmark             Run and track experiments on a number of datasets and configs, and export experiment artifacts.
    serve                 Serves a pretrained model
    visualize             Visualizes experimental results
    collect_summary       Prints names of weights and layers activations to use with other collect commands
@@ -88,6 +91,11 @@ Available sub-commands:
 
         hyperopt_cli.cli(sys.argv[2:])
 
+    def benchmark(self):
+        from ludwig.benchmarking import benchmark
+
+        benchmark.cli(sys.argv[2:])
+
     def serve(self):
         from ludwig import serve
 
