@@ -13,6 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
+INPUT_FEATURES = "input_features"
+OUTPUT_FEATURES = "output_features"
+
+INPUT = "input"
+OUTPUT = "output"
 BINARY = "binary"
 CATEGORY = "category"
 INT = "int"
@@ -36,13 +41,16 @@ INFER_IMAGE_MAX_HEIGHT = "infer_image_max_height"
 INFER_IMAGE_MAX_WIDTH = "infer_image_max_width"
 INFER_IMAGE_SAMPLE_SIZE = "infer_image_sample_size"
 NUM_CHANNELS = "num_channels"
+CLASS_WEIGHTS = "class_weights"
 LOSS = "loss"
 ROC_AUC = "roc_auc"
 EVAL_LOSS = "eval_loss"
 TRAIN_MEAN_LOSS = "train_mean_loss"
+SEQUENCE_SOFTMAX_CROSS_ENTROPY = "sequence_softmax_cross_entropy"
 SOFTMAX_CROSS_ENTROPY = "softmax_cross_entropy"
 SIGMOID_CROSS_ENTROPY = "sigmoid_cross_entropy"
 BINARY_WEIGHTED_CROSS_ENTROPY = "binary_weighted_cross_entropy"
+THRESHOLD = "threshold"
 ACCURACY = "accuracy"
 HITS_AT_K = "hits_at_k"
 MEAN_HITS_AT_K = "mean_hits_at_k"
@@ -58,9 +66,11 @@ EDIT_DISTANCE = "edit_distance"
 PERPLEXITY = "perplexity"
 JACCARD = "jaccard"
 PREDICTIONS = "predictions"
+TOP_K = "top_k"
 TOP_K_PREDICTIONS = "top_k_predictions"
 PROBABILITY = "probability"
 PROBABILITIES = "probabilities"
+SPLIT_PROBABILITIES = "split_probabilities"
 TOKEN_ACCURACY = "token_accuracy"
 LAST_ACCURACY = "last_accuracy"
 SEQUENCE_ACCURACY = "sequence_accuracy"
@@ -75,18 +85,15 @@ FILL_WITH_CONST = "fill_with_const"
 FILL_WITH_MODE = "fill_with_mode"
 FILL_WITH_MEAN = "fill_with_mean"
 FILL_WITH_FALSE = "fill_with_false"
-BACKFILL = "backfill"
 BFILL = "bfill"
-PAD = "pad"
 FFILL = "ffill"
 DROP_ROW = "drop_row"
+MISSING_VALUE_STRATEGY = "missing_value_strategy"
 MISSING_VALUE_STRATEGY_OPTIONS = [
     FILL_WITH_CONST,
     FILL_WITH_MODE,
     FILL_WITH_MEAN,
-    BACKFILL,
     BFILL,
-    PAD,
     FFILL,
     DROP_ROW,
 ]
@@ -104,6 +111,7 @@ LAST_HIDDEN = "last_hidden"
 ENCODER_OUTPUT_STATE = "encoder_output_state"
 PROJECTION_INPUT = "projection_input"
 
+RANDOM = "random"
 SUM = "sum"
 APPEND = "append"
 SEQ_SUM = "seq_sum"
@@ -113,7 +121,13 @@ TRAINING = "training"
 VALIDATION = "validation"
 TEST = "test"
 SPLIT = "split"
+FORCE_SPLIT = "force_split"
+STRATIFY = "stratify"
 FULL = "full"
+TRAIN_SPLIT = 0
+VALIDATION_SPLIT = 1
+TEST_SPLIT = 2
+MIN_VALIDATION_SET_ROWS = 3  # The minimum validation set size to ensure metric computation doesn't fail.
 
 META = "meta"
 
@@ -123,9 +137,12 @@ EXECUTOR = "executor"
 MINIMIZE = "minimize"
 MAXIMIZE = "maximize"
 SAMPLER = "sampler"
+NUM_SAMPLES = "num_samples"
 SEARCH_ALG = "search_alg"
 SCHEDULER = "scheduler"
 PARAMETERS = "parameters"
+GOAL = "goal"
+GRID_SEARCH = "grid_search"
 
 NAME = "name"
 COLUMN = "column"
@@ -142,14 +159,33 @@ PARQUET = "parquet"
 
 SRC = "dataset_src"
 
+EPOCHS = "epochs"
 BATCH_SIZE = "batch_size"
 EVAL_BATCH_SIZE = "eval_batch_size"
+DEFAULT_BATCH_SIZE = 128
 LEARNING_RATE = "learning_rate"
+USE_BIAS = "use_bias"
+BIAS = "bias"
+DEFAULT_USE_BIAS = "default_use_bias"
+DEFAULT_BIAS = "default_bias"
+CONV_USE_BIAS = "conv_use_bias"
+CONV_BIAS = "conv_bias"
 AUTO = "auto"
 CONFIG = "config"
 
+CLIP = "clip"
+DEPENDENCIES = "dependencies"
+REDUCE_INPUT = "reduce_input"
+REDUCE_DEPENDENCIES = "reduce_dependencies"
+
 COMBINER = "combiner"
 
+ENCODER = "encoder"
+DECODER = "decoder"
+
+DEFAULTS = "defaults"
+DEFAULT = "default"
+
 BALANCE_PERCENTAGE_TOLERANCE = 0.03
 IMBALANCE_DETECTION_RATIO = 0.05
 
@@ -172,5 +208,28 @@ HYPEROPT_WARNING = (
 CONTINUE_PROMPT = "Do you want to continue? "
 
 DEFAULT_AUDIO_TENSOR_LENGTH = 70000
+AUDIO_FEATURE_KEYS = [
+    "type",
+    "window_length_in_s",
+    "window_shift_in_s",
+    "num_fft_points",
+    "window_type",
+    "num_filter_bands",
+]
 
+MODEL_TYPE = "model_type"
+MODEL_ECD = "ecd"
+MODEL_GBM = "gbm"
 DASK_MODULE_NAME = "dask.dataframe"
+
+PREPROCESSOR = "preprocessor"
+PREDICTOR = "predictor"
+POSTPROCESSOR = "postprocessor"
+
+S3 = "s3"
+CACHE = "cache"
+
+# If `use_torch_profiler=True` in LudwigProfiler, LUDWIG_TAG is prepended to the specified experiment tag
+# (LudwigProfiler(tag="...", ..)). This edited tag is passed in to `torch.profiler.record_function` so we can
+# retrieve torch ops for the tagged code blocks/functions.
+LUDWIG_TAG = "[ludwig]"
