@@ -14,13 +14,16 @@
 # limitations under the License.
 # ==============================================================================
 import contextlib
+from typing import Iterable, Optional
 
 import numpy as np
+from pandas import DataFrame
 
 from ludwig.constants import PREPROCESSING, TRAINING
 from ludwig.data.batcher.random_access import RandomAccessBatcher
 from ludwig.data.dataset.base import Dataset, DatasetManager
 from ludwig.data.sampler import DistributedSampler
+from ludwig.features.base_feature import BaseFeature
 from ludwig.utils.data_utils import DATA_TRAIN_HDF5_FP, save_hdf5
 from ludwig.utils.dataframe_utils import from_numpy_dataset, to_numpy_dataset
 from ludwig.utils.fs_utils import download_h5
@@ -34,6 +37,12 @@ class PandasDataset(Dataset):
         self.size = len(dataset)
         self.dataset = to_numpy_dataset(dataset)
 
+    def to_df(self, features: Optional[Iterable[BaseFeature]] = None) -> DataFrame:
+        """Convert the dataset to a Pandas DataFrame."""
+        if features:
+            return from_numpy_dataset({feature.feature_name: self.dataset[feature.proc_column] for feature in features})
+        return from_numpy_dataset(self.dataset)
+
     def get(self, proc_column, idx=None):
         if idx is None:
             idx = range(self.size)
@@ -65,15 +74,17 @@ class PandasDataset(Dataset):
     def __len__(self):
         return self.size
 
+    @property
+    def in_memory_size_bytes(self):
+        df = self.to_df()
+        return df.memory_usage(deep=True).sum() if df is not None else 0
+
     @contextlib.contextmanager
     def initialize_batcher(self, batch_size=128, should_shuffle=True, seed=0, ignore_last=False, horovod=None):
         sampler = DistributedSampler(len(self), shuffle=should_shuffle, seed=seed, horovod=horovod)
         batcher = RandomAccessBatcher(self, sampler, batch_size=batch_size, ignore_last=ignore_last)
         yield batcher
 
-    def to_df(self):
-        return from_numpy_dataset(self.dataset)
-
 
 class PandasDatasetManager(DatasetManager):
     def __init__(self, backend):
