@@ -36,12 +36,26 @@ from ludwig.data.dataset.base import Dataset, DatasetManager
 from ludwig.utils.data_utils import DATA_TRAIN_HDF5_FP, DATA_TRAIN_PARQUET_FP
 from ludwig.utils.fs_utils import get_fs_and_path
 from ludwig.utils.misc_utils import get_proc_features
-from ludwig.utils.types import DataFrame
+from ludwig.utils.types import DataFrame, Series
 
 _ray113 = version.parse(ray.__version__) == version.parse("1.13.0")
+_ray_nightly = version.parse(ray.__version__) > version.parse("1.13")
 
 _SCALAR_TYPES = {BINARY, CATEGORY, NUMBER}
 
+# https://github.com/ray-project/ray/issues/27031
+# TODO(geoffrey): remove this once Ray > 1.13 in our CI.
+if _ray_nightly:
+    from ray.data.extensions import TensorArray
+
+    def cast_as_tensor_dtype(series: Series) -> Series:
+        return TensorArray(series)
+
+else:
+
+    def cast_as_tensor_dtype(series: Series) -> Series:
+        return series.astype(TensorDtype())
+
 
 def read_remote_parquet(path: str):
     fs, path = get_fs_and_path(path)
@@ -116,6 +130,12 @@ class RayDataset(Dataset):
     def size(self):
         return len(self)
 
+    @property
+    def in_memory_size_bytes(self):
+        """Memory size may be unknown, so return 0 incase size_bytes() returns None
+        https://docs.ray.io/en/releases-1.12.1/_modules/ray/data/dataset.html#Dataset.size_bytes."""
+        return self.ds.size_bytes() if self.ds is not None else 0
+
     def to_df(self):
         return self.df_engine.from_ray_dataset(self.ds)
 
@@ -267,7 +287,7 @@ class RayDatasetBatcher(Batcher):
             for c in columns:
                 # do not convert scalar columns: https://github.com/ray-project/ray/issues/20825
                 if features[c][TYPE] not in _SCALAR_TYPES:
-                    df[c] = df[c].astype(TensorDtype())
+                    df[c] = cast_as_tensor_dtype(df[c])
                 elif features[c][TYPE] == BINARY:
                     # TODO(travis): figure out why Ray is converting these into object types by default
                     df[c] = df[c].astype(np.bool_)
@@ -276,13 +296,18 @@ class RayDatasetBatcher(Batcher):
         return to_tensors
 
     def _prepare_batch(self, batch: pd.DataFrame) -> Dict[str, np.ndarray]:
-        res = {c: batch[c].to_numpy() for c in self.columns}
+        res = {}
+        for c in self.columns:
+            if self.features[c][TYPE] not in _SCALAR_TYPES:
+                # Ensure columns stacked instead of turned into np.array([np.array, ...], dtype=object) objects
+                res[c] = np.stack(batch[c].values)
+            else:
+                res[c] = batch[c].to_numpy()
 
         for c in self.columns:
             reshape = self.reshape_map.get(c)
             if reshape is not None:
                 res[c] = res[c].reshape((-1, *reshape))
-
         return res
 
     def _create_sync_reader(self, pipeline: DatasetPipeline):
