@@ -14,6 +14,7 @@
 # limitations under the License.
 # ==============================================================================
 import logging
+import warnings
 from abc import ABC, abstractmethod
 from typing import Any, Dict, List, Optional, Tuple
 
@@ -23,12 +24,13 @@ import torch
 
 from ludwig.backend import Backend, LOCAL_BACKEND
 from ludwig.constants import (
-    BACKFILL,
     BFILL,
     BINARY,
     CHECKSUM,
     COLUMN,
+    DEFAULTS,
     DROP_ROW,
+    ENCODER,
     FFILL,
     FILL_WITH_CONST,
     FILL_WITH_FALSE,
@@ -37,7 +39,6 @@ from ludwig.constants import (
     FULL,
     NAME,
     NUMBER,
-    PAD,
     PREPROCESSING,
     PROC_COLUMN,
     SPLIT,
@@ -48,17 +49,22 @@ from ludwig.constants import (
     VALIDATION,
 )
 from ludwig.data.cache.types import wrap
-from ludwig.data.concatenate_datasets import concatenate_df, concatenate_files
+from ludwig.data.concatenate_datasets import concatenate_df, concatenate_files, concatenate_splits
 from ludwig.data.dataset.base import Dataset
+from ludwig.data.split import get_splitter, split_dataset
 from ludwig.encoders.registry import get_encoder_cls
 from ludwig.features.feature_registries import base_type_registry
 from ludwig.features.feature_utils import compute_feature_hash
 from ludwig.utils import data_utils, strings_utils
+from ludwig.utils.backward_compatibility import upgrade_metadata
+from ludwig.utils.config_utils import merge_config_preprocessing_with_feature_specific_defaults
 from ludwig.utils.data_utils import (
     CACHEABLE_FORMATS,
     CSV_FORMATS,
+    DATA_TEST_PARQUET_FP,
     DATA_TRAIN_HDF5_FP,
     DATA_TRAIN_PARQUET_FP,
+    DATA_VALIDATION_PARQUET_FP,
     DATAFRAME_FORMATS,
     DICT_FORMATS,
     EXCEL_FORMATS,
@@ -89,8 +95,6 @@ from ludwig.utils.data_utils import (
     read_stata,
     read_tsv,
     SAS_FORMATS,
-    save_array,
-    split_dataset_ttv,
     SPSS_FORMATS,
     STATA_FORMATS,
     TSV_FORMATS,
@@ -98,9 +102,11 @@ from ludwig.utils.data_utils import (
 )
 from ludwig.utils.defaults import default_preprocessing_parameters, default_random_seed
 from ludwig.utils.fs_utils import file_lock, path_exists
-from ludwig.utils.misc_utils import get_from_registry, merge_dict, resolve_pointers, set_random_seed
+from ludwig.utils.misc_utils import get_from_registry, merge_dict, resolve_pointers
 from ludwig.utils.types import DataFrame, Series
 
+REPARTITIONING_FEATURE_TYPES = {"image", "audio"}
+
 logger = logging.getLogger(__name__)
 
 
@@ -557,7 +563,17 @@ class ParquetPreprocessor(DataFormatPreprocessor):
         random_seed=default_random_seed,
     ):
         test_set = test_set if test_set and path_exists(test_set) else None
+        if test_set and isinstance(test_set, str) and DATA_TEST_PARQUET_FP not in training_set_metadata:
+            training_set_metadata[DATA_TEST_PARQUET_FP] = test_set
+
         validation_set = validation_set if validation_set and path_exists(validation_set) else None
+        if (
+            validation_set
+            and isinstance(validation_set, str)
+            and DATA_VALIDATION_PARQUET_FP not in training_set_metadata
+        ):
+            training_set_metadata[DATA_VALIDATION_PARQUET_FP] = validation_set
+
         if training_set and isinstance(training_set, str) and DATA_TRAIN_PARQUET_FP not in training_set_metadata:
             training_set_metadata[DATA_TRAIN_PARQUET_FP] = training_set
         return training_set, test_set, validation_set, training_set_metadata
@@ -979,7 +995,7 @@ class HDF5Preprocessor(DataFormatPreprocessor):
     @staticmethod
     def preprocess_for_prediction(dataset, features, preprocessing_params, training_set_metadata, backend, callbacks):
         hdf5_fp = dataset
-        dataset = load_hdf5(dataset, features, split_data=False, shuffle_training=False)
+        dataset = load_hdf5(dataset, preprocessing_params, backend, split_data=False, shuffle_training=False)
         return dataset, training_set_metadata, hdf5_fp
 
     @staticmethod
@@ -1020,10 +1036,12 @@ class HDF5Preprocessor(DataFormatPreprocessor):
             training_set_metadata[DATA_TRAIN_HDF5_FP] = not_none_set
 
         if dataset is not None:
-            training_set, test_set, validation_set = load_hdf5(dataset, features, shuffle_training=True)
+            training_set, test_set, validation_set = load_hdf5(
+                dataset, preprocessing_params, backend, shuffle_training=True
+            )
 
         elif training_set is not None:
-            kwargs = dict(features=features, split_data=False)
+            kwargs = dict(preprocessing_params=preprocessing_params, backend=backend, split_data=False)
             training_set = load_hdf5(training_set, shuffle_training=True, **kwargs)
 
             if validation_set is not None:
@@ -1068,6 +1086,27 @@ def build_dataset(
     mode=None,
 ):
     df_engine = backend.df_engine
+
+    if df_engine.partitioned:
+        if any(f["type"] in REPARTITIONING_FEATURE_TYPES for f in features) and dataset_df.npartitions > 1:
+            # A globally unique index only matters if you know that there will be a repartition downstream for some
+            # particular feature, i.e. for Image and Audio features on a Ray backend.
+            # - There is a join operation in `df_like`, and the only way to do the operation is if the partitions across
+            #   all feature columns are aligned.
+            # - In order to align the partitions, we require a way of matching samples to one another across all
+            #   partitions. Therefore, we must reset_index to create a globally unique index.
+            # - If the number of partitions is 1, it is *highly likely* the index is globally unique. Auto-assigned
+            #   Dask indices in this case are unique, and we pd.concat train, val, and test sets with ignore_index=True
+            # If there will NOT be a repartition downstream, then we can skip this step.
+            # - In this case, the partitions should remain aligned throughout.
+            # - Further, while the indices might not be globally unique, they should be unique within each partition.
+            # - These two properties make it possible to do the join op within each partition without a global index.
+            logger.warning(
+                f"Dataset has {dataset_df.npartitions} partitions and feature types that cause repartitioning. "
+                f"Resetting index to ensure globally unique indices."
+            )
+            dataset_df = df_engine.reset_index(dataset_df)
+
     dataset_df = df_engine.parallelize(dataset_df)
 
     sample_ratio = global_preprocessing_parameters["sample_ratio"]
@@ -1100,11 +1139,21 @@ def build_dataset(
         dataset_cols, feature_configs, global_preprocessing_parameters, backend, metadata=metadata
     )
 
-    # Happens after preprocessing parameters are built so we can use precomputed fill values.
+    # Happens after preprocessing parameters are built, so we can use precomputed fill values.
     logger.debug("handle missing values")
+
+    # In some cases, there can be a (temporary) mismatch between the dtype of the column and the type expected by the
+    # preprocessing config (e.g., a categorical feature represented as an int-like column). In particular, Dask
+    # may raise an error even when there are no missing values in the column itself.
+    #
+    # Since we immediately cast all columns in accordance with their expected feature types after filling missing
+    # values, we work around the above issue by temporarily treating all columns as object dtype.
+    for col_key in dataset_cols:
+        dataset_cols[col_key] = dataset_cols[col_key].astype(object)
+
     for feature_config in feature_configs:
         preprocessing_parameters = feature_name_to_preprocessing_parameters[feature_config[NAME]]
-        handle_missing_values(dataset_cols, feature_config, preprocessing_parameters)
+        handle_missing_values(dataset_cols, feature_config, preprocessing_parameters, backend)
 
     # Happens after missing values are handled to avoid NaN casting issues.
     logger.debug("cast columns")
@@ -1130,18 +1179,18 @@ def build_dataset(
     for callback in callbacks or []:
         callback.on_build_data_end(dataset_df, mode)
 
-    logger.debug("get split")
-    split = get_split(
-        dataset_df,
-        force_split=global_preprocessing_parameters["force_split"],
-        split_probabilities=global_preprocessing_parameters["split_probabilities"],
-        stratify=global_preprocessing_parameters["stratify"],
-        backend=backend,
-        random_seed=random_seed,
-    )
+    # Get any additional columns needed for splitting downstream, otherwise they will not be
+    # included in the preprocessed output.
+    split_params = global_preprocessing_parameters.get(SPLIT, {})
+    if "type" not in split_params and SPLIT in dataset_df:
+        warnings.warn(
+            'Detected "split" column in the data, but using default split type '
+            '"random". Did you mean to set split type to "fixed"?'
+        )
 
-    if split is not None:
-        proc_cols[SPLIT] = split
+    splitter = get_splitter(**split_params)
+    for col in splitter.required_columns:
+        proc_cols[col] = dataset_df[col]
 
     # TODO ray: this is needed because ray 1.7 doesn't support Dask to RayDataset
     #  conversion with Tensor columns. Can remove for 1.8.
@@ -1170,6 +1219,12 @@ def build_dataset(
         col_name_to_dtype[col_name] = col.dtype
     dataset = dataset.astype(col_name_to_dtype)
 
+    # Persist the completed dataset with no NaNs
+    dataset = backend.df_engine.persist(dataset)
+
+    # Remove partitions that are empty after removing NaNs
+    dataset = backend.df_engine.remove_empty_partitions(dataset)
+
     return dataset, metadata
 
 
@@ -1220,19 +1275,20 @@ def build_preprocessing_parameters(
         preprocessing_parameters = merge_preprocessing(feature_config, global_preprocessing_parameters)
 
         # deal with encoders that have fixed preprocessing
-        if "encoder" in feature_config:
-            encoder_class = get_encoder_cls(feature_config[TYPE], feature_config["encoder"])
-            if hasattr(encoder_class, "fixed_preprocessing_parameters"):
-                encoder_fpp = encoder_class.fixed_preprocessing_parameters
+        if ENCODER in feature_config:
+            if TYPE in feature_config[ENCODER]:
+                encoder_class = get_encoder_cls(feature_config[TYPE], feature_config[ENCODER][TYPE])
+                if hasattr(encoder_class, "fixed_preprocessing_parameters"):
+                    encoder_fpp = encoder_class.fixed_preprocessing_parameters
 
-                preprocessing_parameters = merge_dict(
-                    preprocessing_parameters, resolve_pointers(encoder_fpp, feature_config, "feature.")
-                )
+                    preprocessing_parameters = merge_dict(
+                        preprocessing_parameters, resolve_pointers(encoder_fpp, feature_config, "feature.")
+                    )
 
         fill_value = precompute_fill_value(dataset_cols, feature_config, preprocessing_parameters, backend)
 
         if fill_value is not None:
-            preprocessing_parameters = {"computed_fill_value": fill_value, **preprocessing_parameters}
+            preprocessing_parameters.update({"computed_fill_value": fill_value})
 
         feature_name_to_preprocessing_parameters[feature_name] = preprocessing_parameters
 
@@ -1288,7 +1344,7 @@ def build_data(
         preprocessing_parameters = training_set_metadata[feature_config[NAME]][PREPROCESSING]
 
         # Need to run this again here as cast_columns may have introduced new missing values
-        handle_missing_values(input_cols, feature_config, preprocessing_parameters)
+        handle_missing_values(input_cols, feature_config, preprocessing_parameters, backend)
 
         get_from_registry(feature_config[TYPE], base_type_registry).add_feature_data(
             feature_config,
@@ -1383,7 +1439,12 @@ def precompute_fill_value(dataset_cols, feature, preprocessing_parameters, backe
         # Distinct values are sorted in reverse to mirror the selection of the default fallback_true_label (in
         # binary_feature.get_feature_meta) for binary columns with unconventional boolean values, "human"/"bot".
         for v in sorted(distinct_values, reverse=True):
-            fallback_true_label = preprocessing_parameters.get("fallback_true_label", "true")
+            fallback_true_label = (
+                preprocessing_parameters["fallback_true_label"]
+                # By default, preprocessing_parameters.fallback_true_label is None.
+                if preprocessing_parameters["fallback_true_label"]
+                else "true"
+            )
             if strings_utils.str2bool(v, fallback_true_label) is False:
                 return v
         raise ValueError(
@@ -1393,68 +1454,42 @@ def precompute_fill_value(dataset_cols, feature, preprocessing_parameters, backe
     return None
 
 
-def handle_missing_values(dataset_cols, feature, preprocessing_parameters):
+def handle_missing_values(dataset_cols, feature, preprocessing_parameters, backend):
     missing_value_strategy = preprocessing_parameters["missing_value_strategy"]
 
     # Check for the precomputed fill value in the metadata
     computed_fill_value = preprocessing_parameters.get("computed_fill_value")
 
-    if computed_fill_value is not None:
+    if (
+        missing_value_strategy in {FILL_WITH_CONST, FILL_WITH_MODE, FILL_WITH_MEAN, FILL_WITH_FALSE}
+        and computed_fill_value is not None
+    ):
         dataset_cols[feature[COLUMN]] = dataset_cols[feature[COLUMN]].fillna(
             computed_fill_value,
         )
-    elif missing_value_strategy in [BACKFILL, BFILL, PAD, FFILL]:
+    elif missing_value_strategy in {BFILL, FFILL}:
         dataset_cols[feature[COLUMN]] = dataset_cols[feature[COLUMN]].fillna(
             method=missing_value_strategy,
         )
+
+        # If the first few rows or last few rows of a dataset is a NaN, it will still be a NaN after ffill or bfill are
+        # applied. This causes downstream errors with Dask (https://github.com/ludwig-ai/ludwig/issues/2452)
+        # To get around this issue, apply the primary missing value strategy (say bfill) first, and then follow it
+        # up with the other missing value strategy (ffill) to ensure all NaNs are filled
+        if backend.df_engine.compute(dataset_cols[feature[COLUMN]].isna().sum()) > 0:
+            dataset_cols[feature[COLUMN]] = dataset_cols[feature[COLUMN]].fillna(
+                method=BFILL if missing_value_strategy == FFILL else FFILL,
+            )
     elif missing_value_strategy == DROP_ROW:
         # Here we only drop from this series, but after preprocessing we'll do a second
         # round of dropping NA values from the entire output dataframe, which will
         # result in the removal of the rows.
         dataset_cols[feature[COLUMN]] = dataset_cols[feature[COLUMN]].dropna()
     else:
-        raise ValueError("Invalid missing value strategy")
+        raise ValueError(f"Invalid missing value strategy {missing_value_strategy}")
 
 
-def get_split(
-    dataset_df,
-    force_split=False,
-    split_probabilities=(0.7, 0.1, 0.2),
-    stratify=None,
-    backend=LOCAL_BACKEND,
-    random_seed=default_random_seed,
-):
-    if SPLIT in dataset_df and not force_split:
-        split = dataset_df[SPLIT].astype(np.int8)
-    else:
-        set_random_seed(random_seed)
-        if stratify is None or stratify not in dataset_df:
-            if backend.df_engine.partitioned:
-                # This approach is very inefficient for partitioned backends, which
-                # can split by partition
-                return
-
-            split = (
-                dataset_df.index.to_series()
-                .map(lambda x: np.random.choice(3, 1, p=split_probabilities))
-                .astype(np.int8)
-            )
-        else:
-            split = np.zeros(len(dataset_df))
-            for val in dataset_df[stratify].unique():
-                # TODO dask: find a way to better parallelize this operation
-                idx_list = dataset_df.index[dataset_df[stratify] == val].tolist()
-                array_lib = backend.df_engine.array_lib
-                val_list = array_lib.random.choice(
-                    3,
-                    len(idx_list),
-                    p=split_probabilities,
-                ).astype(np.int8)
-                split[idx_list] = val_list
-    return split
-
-
-def load_hdf5(hdf5_file_path, features, split_data=True, shuffle_training=False):
+def load_hdf5(hdf5_file_path, preprocessing_params, backend, split_data=True, shuffle_training=False):
     # TODO dask: this needs to work with DataFrames
     logger.info(f"Loading data from: {hdf5_file_path}")
 
@@ -1467,7 +1502,7 @@ def load_hdf5(hdf5_file_path, features, split_data=True, shuffle_training=False)
             dataset = shuffle(dataset)
         return dataset
 
-    training_set, test_set, validation_set = split_dataset_ttv(dataset, SPLIT)
+    training_set, validation_set, test_set = split_dataset(dataset, preprocessing_params, backend)
 
     if shuffle_training:
         training_set = shuffle(training_set)
@@ -1475,9 +1510,13 @@ def load_hdf5(hdf5_file_path, features, split_data=True, shuffle_training=False)
     return training_set, test_set, validation_set
 
 
-def load_metadata(metadata_file_path):
+def load_metadata(metadata_file_path: str) -> Dict[str, Any]:
     logger.info(f"Loading metadata from: {metadata_file_path}")
-    return data_utils.load_json(metadata_file_path)
+    training_set_metadata = data_utils.load_json(metadata_file_path)
+    # TODO(travis): decouple config from training_set_metadata so we don't need to
+    #  upgrade it over time.
+    training_set_metadata = upgrade_metadata(training_set_metadata)
+    return training_set_metadata
 
 
 def preprocess_for_training(
@@ -1608,16 +1647,32 @@ def preprocess_for_training(
         with use_credentials(backend.cache.credentials if cached else None):
             logger.debug("create training dataset")
             training_dataset = backend.dataset_manager.create(training_set, config, training_set_metadata)
+            if not len(training_set):
+                raise ValueError("Training data is empty following preprocessing.")
 
             validation_dataset = None
             if validation_set is not None:
                 logger.debug("create validation dataset")
                 validation_dataset = backend.dataset_manager.create(validation_set, config, training_set_metadata)
+                if not len(validation_dataset):
+                    # Validation dataset is empty.
+                    logger.warning(
+                        "Encountered empty validation dataset. If this is unintentional, please check the "
+                        "preprocessing configuration."
+                    )
+                    validation_dataset = None
 
             test_dataset = None
             if test_set is not None:
                 logger.debug("create test dataset")
                 test_dataset = backend.dataset_manager.create(test_set, config, training_set_metadata)
+                if not len(test_dataset):
+                    # Test dataset is empty.
+                    logger.warning(
+                        "Encountered empty test dataset. If this is unintentional, please check the "
+                        "preprocessing configuration."
+                    )
+                    test_dataset = None
 
         return (training_dataset, validation_dataset, test_dataset, training_set_metadata)
 
@@ -1672,12 +1727,6 @@ def _preprocess_file_for_training(
             mode="training",
         )
 
-        # TODO(travis): implement saving split for Ray
-        if backend.is_coordinator() and not skip_save_processed_input and SPLIT in data.columns:
-            # save split values for use by visualization routines
-            split_fp = get_split_path(dataset)
-            save_array(split_fp, data[SPLIT])
-
     elif training_set:
         # use data_train (including _validation and _test if they are present)
         # and ignore data and train set metadata
@@ -1688,6 +1737,21 @@ def _preprocess_file_for_training(
         concatenated_df = concatenate_files(training_set, validation_set, test_set, read_fn, backend)
         training_set_metadata[SRC] = training_set
 
+        # Data is pre-split, so we override whatever split policy the user specified
+        if preprocessing_params["split"]:
+            warnings.warn(
+                'Preprocessing "split" section provided, but pre-split dataset given as input. '
+                "Ignoring split configuration."
+            )
+
+        preprocessing_params = {
+            **preprocessing_params,
+            "split": {
+                "type": "fixed",
+                "column": SPLIT,
+            },
+        }
+
         data, training_set_metadata = build_dataset(
             concatenated_df,
             features,
@@ -1702,15 +1766,16 @@ def _preprocess_file_for_training(
     else:
         raise ValueError("either data or data_train have to be not None")
 
+    logger.debug("split train-val-test")
+    training_data, validation_data, test_data = split_dataset(data, preprocessing_params, backend, random_seed)
+
+    if dataset and backend.is_coordinator() and not skip_save_processed_input:
+        logger.debug("writing split file")
+        splits_df = concatenate_splits(training_data, validation_data, test_data, backend)
+        split_fp = get_split_path(dataset or training_set)
+        backend.df_engine.to_parquet(splits_df, split_fp, index=True)
+
     logger.info("Building dataset: DONE")
-
-    if SPLIT in data.columns:
-        logger.debug("split on split column")
-        training_data, test_data, validation_data = split_dataset_ttv(data, SPLIT)
-    else:
-        logger.debug("split randomly by partition")
-        training_data, test_data, validation_data = data.random_split(preprocessing_params["split_probabilities"])
-
     if preprocessing_params["oversample_minority"] or preprocessing_params["undersample_majority"]:
         training_data = balance_data(training_data, config["output_features"], preprocessing_params, backend)
 
@@ -1742,9 +1807,25 @@ def _preprocess_df_for_training(
         # needs preprocessing
         logger.info("Using training dataframe")
         dataset = concatenate_df(training_set, validation_set, test_set, backend)
+
+        # Data is pre-split, so we override whatever split policy the user specified
+        if preprocessing_params["split"]:
+            warnings.warn(
+                'Preprocessing "split" section provided, but pre-split dataset given as input. '
+                "Ignoring split configuration."
+            )
+
+        preprocessing_params = {
+            **preprocessing_params,
+            "split": {
+                "type": "fixed",
+                "column": SPLIT,
+            },
+        }
+
     logger.info("Building dataset (it may take a while)")
 
-    dataset, training_set_metadata = build_dataset(
+    data, training_set_metadata = build_dataset(
         dataset,
         features,
         preprocessing_params,
@@ -1755,15 +1836,10 @@ def _preprocess_df_for_training(
         mode="training",
     )
 
+    logger.debug("split train-val-test")
+    training_set, validation_set, test_set = split_dataset(data, preprocessing_params, backend, random_seed)
+
     logger.info("Building dataset: DONE")
-
-    if SPLIT in dataset.columns:
-        logger.debug("split on split column")
-        training_set, test_set, validation_set = split_dataset_ttv(dataset, SPLIT)
-    else:
-        logger.debug("split randomly by partition")
-        training_set, test_set, validation_set = dataset.random_split(preprocessing_params["split_probabilities"])
-
     if preprocessing_params["oversample_minority"] or preprocessing_params["undersample_majority"]:
         training_set = balance_data(training_set, config["output_features"], preprocessing_params, backend)
 
@@ -1807,7 +1883,10 @@ def preprocess_for_prediction(
         if num_overrides > 0:
             logger.warning("Using in_memory = False is not supported " "with {} data format.".format(data_format))
 
-    preprocessing_params = merge_dict(default_preprocessing_parameters, config[PREPROCESSING])
+    preprocessing_params = merge_config_preprocessing_with_feature_specific_defaults(
+        config.get(PREPROCESSING, {}), config.get(DEFAULTS, {})
+    )
+    preprocessing_params = merge_dict(default_preprocessing_parameters, preprocessing_params)
 
     # if training_set_metadata is a string, assume it's a path to load the json
     if training_set_metadata and isinstance(training_set_metadata, str):
@@ -1873,7 +1952,8 @@ def preprocess_for_prediction(
             training_set_metadata[DATA_TRAIN_HDF5_FP] = new_hdf5_fp
 
         if split != FULL:
-            training_set, test_set, validation_set = split_dataset_ttv(dataset, SPLIT)
+            logger.debug("split train-val-test")
+            training_set, validation_set, test_set = split_dataset(dataset, preprocessing_params, backend)
 
     if split == TRAINING:
         dataset = training_set
