@@ -1,7 +1,7 @@
 from typing import Dict, List, Type, Union
 
 from ludwig.encoders.base import Encoder
-from ludwig.utils.registry import DEFAULT_KEYS, Registry
+from ludwig.utils.registry import Registry
 
 encoder_registry = Registry()
 
@@ -16,7 +16,7 @@ def register_sequence_encoder(name: str):
     return wrap
 
 
-def register_encoder(name: str, features: Union[str, List[str]], default=False):
+def register_encoder(name: str, features: Union[str, List[str]]):
     if isinstance(features, str):
         features = [features]
 
@@ -24,9 +24,6 @@ def register_encoder(name: str, features: Union[str, List[str]], default=False):
         for feature in features:
             feature_registry = encoder_registry.get(feature, {})
             feature_registry[name] = cls
-            if default:
-                for key in DEFAULT_KEYS:
-                    feature_registry[key] = cls
             encoder_registry[feature] = feature_registry
         return cls
 
