@@ -14,16 +14,19 @@
 # limitations under the License.
 # ==============================================================================
 import logging
-from datetime import date, datetime
-from typing import Any, Dict, List
+from datetime import datetime
+from typing import Any, Dict, List, Union
 
 import numpy as np
 import torch
 from dateutil.parser import parse
 
-from ludwig.constants import COLUMN, DATE, FILL_WITH_CONST, MISSING_VALUE_STRATEGY_OPTIONS, PROC_COLUMN, TIED
+from ludwig.constants import COLUMN, DATE, ENCODER, PROC_COLUMN, TIED, TYPE
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature
-from ludwig.utils.misc_utils import set_default_value
+from ludwig.schema.features.date_feature import DateInputFeatureConfig
+from ludwig.schema.features.utils import register_input_feature
+from ludwig.utils.date_utils import create_vector_from_datetime_obj
+from ludwig.utils.misc_utils import set_default_value, set_default_values
 from ludwig.utils.types import DataFrame, TorchscriptPreprocessingInput
 
 logger = logging.getLogger(__name__)
@@ -52,16 +55,7 @@ class DateFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def preprocessing_defaults():
-        return {"missing_value_strategy": FILL_WITH_CONST, "fill_value": "", "datetime_format": None}
-
-    @staticmethod
-    def preprocessing_schema():
-        return {
-            "missing_value_strategy": {"type": "string", "enum": MISSING_VALUE_STRATEGY_OPTIONS},
-            "fill_value": {"type": "string"},
-            "computed_fill_value": {"type": "string"},
-            "datetime_format": {"type": ["string", "null"]},
-        }
+        return DateInputFeatureConfig().preprocessing.to_dict()
 
     @staticmethod
     def cast_column(column, backend):
@@ -74,19 +68,21 @@ class DateFeatureMixin(BaseFeatureMixin):
     @staticmethod
     def date_to_list(date_str, datetime_format, preprocessing_parameters):
         try:
-            if datetime_format is not None:
+            if isinstance(date_str, datetime):
+                datetime_obj = date_str
+            elif datetime_format is not None:
                 datetime_obj = datetime.strptime(date_str, datetime_format)
             else:
                 datetime_obj = parse(date_str)
         except Exception as e:
-            logging.error(
+            logger.error(
                 f"Error parsing date: {date_str} with error {e} "
                 "Please provide a datetime format that parses it "
                 "in the preprocessing section of the date feature "
                 "in the config. "
                 "The preprocessing fill in value will be used."
                 "For more details: "
-                "https://ludwig-ai.github.io/ludwig-docs/0.5/configuration/features/date_features/"
+                "https://ludwig-ai.github.io/ludwig-docs/latest/configuration/features/date_features/#date-features-preprocessing"  # noqa
             )
             fill_value = preprocessing_parameters["fill_value"]
             if fill_value != "":
@@ -96,6 +92,7 @@ class DateFeatureMixin(BaseFeatureMixin):
 
         return create_vector_from_datetime_obj(datetime_obj)
 
+    @staticmethod
     def add_feature_data(
         feature_config: Dict[str, Any],
         input_df: DataFrame,
@@ -115,16 +112,16 @@ class DateFeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
+@register_input_feature(DATE)
 class DateInputFeature(DateFeatureMixin, InputFeature):
-    encoder = "embed"
+    def __init__(self, input_feature_config: Union[DateInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
+        input_feature_config = self.load_config(input_feature_config)
+        super().__init__(input_feature_config, **kwargs)
 
-    def __init__(self, feature, encoder_obj=None):
-        super().__init__(feature)
-        self.overwrite_defaults(feature)
         if encoder_obj:
             self.encoder_obj = encoder_obj
         else:
-            self.encoder_obj = self.initialize_encoder(feature)
+            self.encoder_obj = self.initialize_encoder(input_feature_config.encoder)
 
     def forward(self, inputs):
         assert isinstance(inputs, torch.Tensor)
@@ -153,27 +150,14 @@ class DateInputFeature(DateFeatureMixin, InputFeature):
 
     @staticmethod
     def populate_defaults(input_feature):
-        set_default_value(input_feature, TIED, None)
+        defaults = DateInputFeatureConfig()
+        set_default_value(input_feature, TIED, defaults.tied)
+        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
+
+    @staticmethod
+    def get_schema_cls():
+        return DateInputFeatureConfig
 
     @staticmethod
     def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
         return _DatePreprocessing(metadata)
-
-
-def create_vector_from_datetime_obj(datetime_obj):
-    yearday = datetime_obj.toordinal() - date(datetime_obj.year, 1, 1).toordinal() + 1
-
-    midnight = datetime_obj.replace(hour=0, minute=0, second=0, microsecond=0)
-    second_of_day = (datetime_obj - midnight).seconds
-
-    return [
-        datetime_obj.year,
-        datetime_obj.month,
-        datetime_obj.day,
-        datetime_obj.weekday(),
-        yearday,
-        datetime_obj.hour,
-        datetime_obj.minute,
-        datetime_obj.second,
-        second_of_day,
-    ]
