@@ -100,7 +100,22 @@ def sanitize(name):
 
 
 def compute_feature_hash(feature: dict) -> str:
-    preproc_hash = hash_dict(feature.get(PREPROCESSING, {}))
+    """
+    This function computes a hash for each feature based on the preprocessing dictionary associated with each feature.
+    The input is always the feature dict, however sometimes, this is called from BaseFeature which dumps the feature
+    dict from a ConfigObject and the preprocessing key corresponds to a nested Preprocessing config. This is why it
+    includes the if/else statement.
+    Args:
+        feature: Feature dictionary
+
+    Returns: Feature hash name
+
+    """
+    preproc = feature.get(PREPROCESSING, {})
+    if isinstance(preproc, dict):
+        preproc_hash = hash_dict(preproc)
+    else:
+        preproc_hash = hash_dict(preproc.to_dict())
     return sanitize(feature[NAME]) + "_" + preproc_hash.decode("ascii")
 
 
@@ -116,7 +131,7 @@ def get_input_size_with_dependencies(
     """
     input_size_with_dependencies = combiner_output_size
     for feature_name in dependencies:
-        if other_output_features[feature_name].num_fc_layers:
+        if other_output_features[feature_name].fc_stack.num_layers:
             input_size_with_dependencies += other_output_features[feature_name].fc_stack.output_shape[-1]
         else:
             # 0-layer FCStack. Use the output feature's input size.
