@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,15 +13,17 @@
 # limitations under the License.
 # ==============================================================================
 import logging
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Union
 
 import numpy as np
 import torch
 
-from ludwig.constants import COLUMN, FILL_WITH_CONST, H3, MISSING_VALUE_STRATEGY_OPTIONS, PROC_COLUMN, TIED
+from ludwig.constants import COLUMN, ENCODER, H3, PROC_COLUMN, TIED, TYPE
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature
+from ludwig.schema.features.h3_feature import H3InputFeatureConfig
+from ludwig.schema.features.utils import register_input_feature
 from ludwig.utils.h3_util import h3_to_components
-from ludwig.utils.misc_utils import set_default_value
+from ludwig.utils.misc_utils import set_default_value, set_default_values
 from ludwig.utils.types import TorchscriptPreprocessingInput
 
 logger = logging.getLogger(__name__)
@@ -59,7 +60,7 @@ class _H3Preprocessing(torch.nn.Module):
                 components.base_cell,
             ]
             cells_padding: List[int] = [self.h3_padding_value] * (self.max_h3_resolution - len(components.cells))
-            output = torch.tensor(header + components.cells + cells_padding, dtype=torch.uint8)
+            output = torch.tensor(header + components.cells + cells_padding, dtype=torch.uint8, device=v.device)
             outputs.append(output)
 
         return torch.stack(outputs)
@@ -72,26 +73,14 @@ class H3FeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def preprocessing_defaults():
-        return {
-            "missing_value_strategy": FILL_WITH_CONST,
-            "fill_value": 576495936675512319
-            # mode 1 edge 0 resolution 0 base_cell 0
-        }
-
-    @staticmethod
-    def preprocessing_schema():
-        return {
-            "missing_value_strategy": {"type": "string", "enum": MISSING_VALUE_STRATEGY_OPTIONS},
-            "fill_value": {"type": "integer"},
-            "computed_fill_value": {"type": "integer"},
-        }
+        return H3InputFeatureConfig().preprocessing.to_dict()
 
     @staticmethod
     def cast_column(column, backend):
         try:
             return column.astype(int)
         except ValueError:
-            logging.warning("H3Feature could not be read as int directly. Reading as float and converting to int.")
+            logger.warning("H3Feature could not be read as int directly. Reading as float and converting to int.")
             return column.astype(float).astype(int)
 
     @staticmethod
@@ -111,8 +100,8 @@ class H3FeatureMixin(BaseFeatureMixin):
     ):
         column = input_df[feature_config[COLUMN]]
         if column.dtype == object:
-            column = column.map(int)
-        column = column.map(H3FeatureMixin.h3_to_list)
+            column = backend.df_engine.map_objects(column, int)
+        column = backend.df_engine.map_objects(column, H3FeatureMixin.h3_to_list)
 
         proc_df[feature_config[PROC_COLUMN]] = backend.df_engine.map_objects(
             column, lambda x: np.array(x, dtype=np.uint8)
@@ -120,16 +109,16 @@ class H3FeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
+@register_input_feature(H3)
 class H3InputFeature(H3FeatureMixin, InputFeature):
-    encoder = "embed"
+    def __init__(self, input_feature_config: Union[H3InputFeatureConfig, Dict], encoder_obj=None, **kwargs):
+        input_feature_config = self.load_config(input_feature_config)
+        super().__init__(input_feature_config, **kwargs)
 
-    def __init__(self, feature, encoder_obj=None):
-        super().__init__(feature)
-        self.overwrite_defaults(feature)
         if encoder_obj:
             self.encoder_obj = encoder_obj
         else:
-            self.encoder_obj = self.initialize_encoder(feature)
+            self.encoder_obj = self.initialize_encoder(input_feature_config.encoder)
 
     def forward(self, inputs):
         assert isinstance(inputs, torch.Tensor)
@@ -158,8 +147,14 @@ class H3InputFeature(H3FeatureMixin, InputFeature):
 
     @staticmethod
     def populate_defaults(input_feature):
-        set_default_value(input_feature, TIED, None)
+        defaults = H3InputFeatureConfig()
+        set_default_value(input_feature, TIED, defaults.tied)
+        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
 
     @staticmethod
     def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
         return _H3Preprocessing(metadata)
+
+    @staticmethod
+    def get_schema_cls():
+        return H3InputFeatureConfig
