@@ -14,9 +14,8 @@
 # limitations under the License.
 # ==============================================================================
 
-LUDWIG_VERSION = "0.5.5"
+LUDWIG_VERSION = "0.6"
 
-INFERENCE_MODULE_FILE_NAME = "inference_module"
 MODEL_WEIGHTS_FILE_NAME = "model_weights"
 MODEL_HYPERPARAMETERS_FILE_NAME = "model_hyperparameters.json"
 TRAIN_SET_METADATA_FILE_NAME = "training_set_metadata.json"
@@ -25,11 +24,17 @@ TRAINING_CHECKPOINTS_DIR_PATH = "training_checkpoints"
 
 TEST_STATISTICS_FILE_NAME = "test_statistics.json"
 
+DESCRIPTION_FILE_NAME = "description.json"
+
 PREDICTIONS_PARQUET_FILE_NAME = "predictions.parquet"
 PREDICTIONS_SHAPES_FILE_NAME = "predictions.shapes.json"
 
 TRAINING_PREPROC_FILE_NAME = "training.hdf5"
 
+HYPEROPT_STATISTICS_FILE_NAME = "hyperopt_statistics.json"
+
+CONFIG_YAML = "config.yaml"
+
 DISABLE_PROGRESSBAR = False
 
 
