@@ -1,3 +1,4 @@
+import contextlib
 import copy
 import datetime
 import glob
@@ -14,49 +15,44 @@ from inspect import signature
 from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
+import ray
 from packaging import version
+from ray import tune
+from ray.tune import ExperimentAnalysis, register_trainable, Stopper
+from ray.tune.schedulers.resource_changing_scheduler import DistributeResources, ResourceChangingScheduler
+from ray.tune.suggest import BasicVariantGenerator, ConcurrencyLimiter
+from ray.tune.utils import wait_for_gpu
+from ray.tune.utils.placement_groups import PlacementGroupFactory
+from ray.util.queue import Queue as RayQueue
 
 from ludwig.api import LudwigModel
 from ludwig.backend import initialize_backend, RAY
+from ludwig.backend.ray import initialize_ray
 from ludwig.callbacks import Callback
-from ludwig.constants import COLUMN, MAXIMIZE, TEST, TRAINER, TRAINING, TYPE, VALIDATION
-from ludwig.hyperopt.results import RayTuneResults, TrialResults
+from ludwig.constants import MAXIMIZE, TEST, TRAINER, TRAINING, TYPE, VALIDATION
+from ludwig.hyperopt.results import HyperoptResults, TrialResults
 from ludwig.hyperopt.search_algos import get_search_algorithm
-from ludwig.hyperopt.utils import load_json_values
+from ludwig.hyperopt.utils import load_json_values, substitute_parameters
 from ludwig.modules.metric_modules import get_best_function
 from ludwig.utils import metric_utils
 from ludwig.utils.data_utils import hash_dict, NumpyEncoder
-from ludwig.utils.defaults import default_random_seed
+from ludwig.utils.defaults import default_random_seed, merge_with_defaults
 from ludwig.utils.fs_utils import has_remote_protocol
 from ludwig.utils.misc_utils import get_from_registry
 
+_ray_112 = version.parse("1.12") <= version.parse(ray.__version__) < version.parse("1.13")
+_ray_113 = version.parse(ray.__version__) >= version.parse("1.13")
+_ray_200 = version.parse(ray.__version__) >= version.parse("2.0")
+if _ray_200:
+    from ray.tune.search import SEARCH_ALG_IMPORT
+    from ray.tune.syncer import get_node_to_storage_syncer, SyncConfig
+else:
+    from ray.tune.suggest import SEARCH_ALG_IMPORT
+    from ray.tune.syncer import get_cloud_sync_client
+
+
 logger = logging.getLogger(__name__)
 
-try:
-    import ray
-    from ray import tune
-    from ray.tune import register_trainable, Stopper
-    from ray.tune.schedulers.resource_changing_scheduler import DistributeResources, ResourceChangingScheduler
-    from ray.tune.suggest import BasicVariantGenerator, ConcurrencyLimiter
-
-    _ray_114 = version.parse(ray.__version__) >= version.parse("1.14")
-    if _ray_114:
-        from ray.tune.search import SEARCH_ALG_IMPORT
-        from ray.tune.syncer import get_node_to_storage_syncer, SyncConfig
-    else:
-        from ray.tune.syncer import get_cloud_sync_client
-        from ray.tune.suggest import SEARCH_ALG_IMPORT
-
-    from ray.tune.utils import wait_for_gpu
-    from ray.tune.utils.placement_groups import PlacementGroupFactory
-    from ray.util.queue import Queue as RayQueue
-
-except ImportError as e:
-    logger.warning(f"ImportError (execution.py) failed to import ray with error: \n\t{e}")
-    ray = None
-    Stopper = object
-    get_horovod_kwargs = None
-
 
 try:
     from ludwig.backend.ray import RayBackend
@@ -152,12 +148,7 @@ class RayTuneExecutor:
         self.output_feature = output_feature
         self.metric = metric
         self.split = split
-        if not ray.is_initialized():
-            try:
-                ray.init("auto", ignore_reinit_error=True)
-            except ConnectionError:
-                logger.info("Initializing new Ray cluster...")
-                ray.init(ignore_reinit_error=True)
+        initialize_ray()
         self.search_space, self.decode_ctx = self._get_search_space(parameters)
         self.num_samples = num_samples
         self.goal = goal
@@ -362,20 +353,33 @@ class RayTuneExecutor:
                 # Remove checkpoint marker on incomplete directory
                 os.remove(marker_path)
 
-    def _get_best_model_path(self, trial_path, analysis):
+    @contextlib.contextmanager
+    def _get_best_model_path(self, trial_path: str, analysis: ExperimentAnalysis) -> str:
         remote_checkpoint_dir = self._get_remote_checkpoint_dir(Path(trial_path))
         if remote_checkpoint_dir is not None:
             self.sync_client.sync_down(remote_checkpoint_dir, trial_path)
             self.sync_client.wait_or_retry()
         self._remove_partial_checkpoints(trial_path)  # needed by get_best_checkpoint
-        mod_path = None
+
         try:
-            mod_path = analysis.get_best_checkpoint(trial_path.rstrip("/"))
+            checkpoint = analysis.get_best_checkpoint(trial_path.rstrip("/"))
         except Exception:
             logger.warning(
                 f"Cannot get best model path for {trial_path} due to exception below:\n{traceback.format_exc()}"
             )
-        return mod_path
+            yield None
+            return
+
+        if _ray_112 and checkpoint is not None:
+            # In Ray 1.12 and 1.12.1, checkpoints don't have an as_directory() context manager so
+            # return the local path from the TrialCheckpoint object
+            yield checkpoint._local_path
+        elif _ray_113 and checkpoint is not None:
+            # In Ray 1.13, checkpoints have changed from strings to objects
+            with checkpoint.as_directory() as path:
+                yield path
+        else:
+            yield checkpoint
 
     @staticmethod
     def _evaluate_best_model(
@@ -426,19 +430,33 @@ class RayTuneExecutor:
                 f"{traceback.format_exc()}"
             )
 
-    def _run_experiment(self, config, checkpoint_dir, hyperopt_dict, decode_ctx, is_using_ray_backend=False):
+    def _run_experiment(
+        self,
+        config,
+        checkpoint_dir,
+        hyperopt_dict,
+        decode_ctx,
+        is_using_ray_backend=False,
+    ):
         for gpu_id in ray.get_gpu_ids():
             # Previous trial may not have freed its memory yet, so wait to avoid OOM
             wait_for_gpu(gpu_id)
+
         # Some config values may be JSON encoded as strings, so decode them here
         config = self.decode_values(config, decode_ctx)
 
-        trial_id = tune.get_trial_id()
-        modified_config = substitute_parameters(copy.deepcopy(hyperopt_dict["config"]), config)
+        # Remove mlflow injected config parameters: https://github.com/ludwig-ai/ludwig/issues/2288
+        if "mlflow" in config:
+            del config["mlflow"]
 
+        trial_id = tune.get_trial_id()
         trial_dir = Path(tune.get_trial_dir())
         driver_trial_location = ray.util.get_node_ip_address()
 
+        modified_config = substitute_parameters(copy.deepcopy(hyperopt_dict["config"]), config)
+
+        modified_config = merge_with_defaults(modified_config)
+
         hyperopt_dict["config"] = modified_config
         hyperopt_dict["experiment_name "] = f'{hyperopt_dict["experiment_name"]}_{trial_id}'
         hyperopt_dict["output_directory"] = str(trial_dir)
@@ -631,7 +649,7 @@ class RayTuneExecutor:
         debug=False,
         hyperopt_log_verbosity=3,
         **kwargs,
-    ) -> RayTuneResults:
+    ) -> HyperoptResults:
         if isinstance(dataset, str) and not has_remote_protocol(dataset) and not os.path.isabs(dataset):
             dataset = os.path.abspath(dataset)
 
@@ -721,7 +739,11 @@ class RayTuneExecutor:
 
         def run_experiment_trial(config, local_hyperopt_dict, checkpoint_dir=None):
             return self._run_experiment(
-                config, checkpoint_dir, local_hyperopt_dict, self.decode_ctx, _is_ray_backend(backend)
+                config,
+                checkpoint_dir,
+                local_hyperopt_dict,
+                self.decode_ctx,
+                _is_ray_backend(backend),
             )
 
         tune_config = {}
@@ -745,7 +767,7 @@ class RayTuneExecutor:
         if has_remote_protocol(output_directory):
             run_experiment_trial = tune.durable(run_experiment_trial)
             self.sync_config = tune.SyncConfig(sync_to_driver=False, upload_dir=output_directory)
-            if _ray_114:
+            if _ray_200:
                 self.sync_client = get_node_to_storage_syncer(SyncConfig(upload_dir=output_directory))
             else:
                 self.sync_client = get_cloud_sync_client(output_directory)
@@ -757,7 +779,12 @@ class RayTuneExecutor:
             self.sync_client = KubernetesSyncClient(self.kubernetes_namespace)
 
         run_experiment_trial_params = tune.with_parameters(run_experiment_trial, local_hyperopt_dict=hyperopt_dict)
-        register_trainable(f"trainable_func_f{hash_dict(config).decode('ascii')}", run_experiment_trial_params)
+
+        @ray.remote
+        def _register(name, trainable):
+            register_trainable(name, trainable)
+
+        ray.get(_register.remote(f"trainable_func_f{hash_dict(config).decode('ascii')}", run_experiment_trial_params))
 
         # Note that resume="AUTO" will attempt to resume the experiment if possible, and
         # otherwise will start a new experiment:
@@ -817,25 +844,25 @@ class RayTuneExecutor:
                     # Evaluate the best model on the eval_split, which is validation_set
                     if validation_set is not None and validation_set.size > 0:
                         trial_path = trial["trial_dir"]
-                        best_model_path = self._get_best_model_path(trial_path, analysis)
-                        if best_model_path is not None:
-                            self._evaluate_best_model(
-                                trial,
-                                trial_path,
-                                best_model_path,
-                                validation_set,
-                                data_format,
-                                skip_save_unprocessed_output,
-                                skip_save_predictions,
-                                skip_save_eval_stats,
-                                gpus,
-                                gpu_memory_limit,
-                                allow_parallel_threads,
-                                backend,
-                                debug,
-                            )
-                        else:
-                            logger.warning("Skipping evaluation as no model checkpoints were available")
+                        with self._get_best_model_path(trial_path, analysis) as best_model_path:
+                            if best_model_path is not None:
+                                self._evaluate_best_model(
+                                    trial,
+                                    trial_path,
+                                    best_model_path,
+                                    validation_set,
+                                    data_format,
+                                    skip_save_unprocessed_output,
+                                    skip_save_predictions,
+                                    skip_save_eval_stats,
+                                    gpus,
+                                    gpu_memory_limit,
+                                    allow_parallel_threads,
+                                    backend,
+                                    debug,
+                                )
+                            else:
+                                logger.warning("Skipping evaluation as no model checkpoints were available")
                     else:
                         logger.warning("Skipping evaluation as no validation set was provided")
 
@@ -844,7 +871,7 @@ class RayTuneExecutor:
             logger.warning("No trials reported results; check if time budget lower than epoch latency")
             ordered_trials = []
 
-        return RayTuneResults(ordered_trials=ordered_trials, experiment_analysis=analysis)
+        return HyperoptResults(ordered_trials=ordered_trials, experiment_analysis=analysis)
 
 
 class CallbackStopper(Stopper):
@@ -870,42 +897,15 @@ def get_build_hyperopt_executor(executor_type):
 executor_registry = {"ray": RayTuneExecutor}
 
 
-def set_values(model_dict, name, parameters_dict):
-    if name in parameters_dict:
-        params = parameters_dict[name]
-        for key, value in params.items():
-            if isinstance(value, dict):
-                for sub_key, sub_value in value.items():
-                    model_dict[key][sub_key] = sub_value
-            else:
-                model_dict[key] = value
-
-
-def get_parameters_dict(parameters):
-    parameters_dict = {}
-    for name, value in parameters.items():
-        curr_dict = parameters_dict
-        name_list = name.split(".")
-        for i, name_elem in enumerate(name_list):
-            if i == len(name_list) - 1:
-                curr_dict[name_elem] = value
-            else:
-                name_dict = curr_dict.get(name_elem, {})
-                curr_dict[name_elem] = name_dict
-                curr_dict = name_dict
-    return parameters_dict
-
-
-def substitute_parameters(config, parameters):
-    parameters_dict = get_parameters_dict(parameters)
-    for input_feature in config["input_features"]:
-        set_values(input_feature, input_feature[COLUMN], parameters_dict)
-    for output_feature in config["output_features"]:
-        set_values(output_feature, output_feature[COLUMN], parameters_dict)
-    set_values(config["combiner"], "combiner", parameters_dict)
-    set_values(config[TRAINER], TRAINER, parameters_dict)
-    set_values(config["preprocessing"], "preprocessing", parameters_dict)
-    return config
+def set_values(params: Dict[str, Any], model_dict: Dict[str, Any]):
+    for key, value in params.items():
+        if isinstance(value, dict):
+            for sub_key, sub_value in value.items():
+                if key not in model_dict:
+                    model_dict[key] = dict()
+                model_dict[key][sub_key] = sub_value
+        else:
+            model_dict[key] = value
 
 
 def run_experiment(
@@ -953,6 +953,7 @@ def run_experiment(
         allow_parallel_threads=allow_parallel_threads,
         callbacks=callbacks,
     )
+
     eval_stats, train_stats, _, _ = model.experiment(
         dataset=dataset,
         training_set=training_set,
