@@ -1,11 +1,12 @@
 from abc import ABC
 from dataclasses import field
-from typing import ClassVar, Optional, Tuple
+from typing import ClassVar, Dict, Optional, Tuple
 
 import torch
 from marshmallow import fields, ValidationError
 from marshmallow_dataclass import dataclass
 
+from ludwig.schema.metadata.trainer_metadata import TRAINER_METADATA
 from ludwig.schema.utils import (
     BaseMarshmallowConfig,
     Boolean,
@@ -18,6 +19,8 @@ from ludwig.schema.utils import (
 )
 from ludwig.utils.registry import Registry
 
+from .metadata.parameter_metadata import convert_metadata_to_json
+
 optimizer_registry = Registry()
 
 
@@ -40,11 +43,14 @@ class BaseOptimizerConfig(BaseMarshmallowConfig, ABC):
 
     optimizer_class: ClassVar[Optional[torch.optim.Optimizer]] = None
     "Class variable pointing to the corresponding `torch.optim.Optimizer` class."
+
     type: str
     """Name corresponding to an optimizer `ludwig.modules.optimization_modules.optimizer_registry`.
        Technically mutable, but attempting to load a derived optimizer with `type` set to a mismatched value will
        result in a `ValidationError`."""
 
+    lr: float = NonNegativeFloat(default=1e-03, description="Learning rate.")
+
 
 @register_optimizer(name="sgd")
 @dataclass
@@ -58,7 +64,7 @@ class SGDOptimizerConfig(BaseOptimizerConfig):
     """Must be 'sgd' - corresponds to name in `ludwig.modules.optimization_modules.optimizer_registry` (default:
        'sgd')"""
 
-    lr: float = FloatRange(default=1e-03, min=0.0, max=1.0, description="Learning rate.")
+    lr: float = NonNegativeFloat(default=1e-03, description="Learning rate.")
 
     # Defaults taken from https://pytorch.org/docs/stable/generated/torch.optim.SGD.html#torch.optim.SGD :
     momentum: float = NonNegativeFloat(default=0.0, description="Momentum factor.")
@@ -80,14 +86,18 @@ class AdamOptimizerConfig(BaseOptimizerConfig):
        (default: 'adam')"""
 
     # Defaults taken from https://pytorch.org/docs/stable/generated/torch.optim.Adam.html#torch.optim.Adam :
-    lr: float = FloatRange(default=1e-03, min=0.0, max=1.0, description="Learning rate.")
+    lr: float = NonNegativeFloat(default=1e-03, description="Learning rate.")
+
     betas: Tuple[float, float] = FloatRangeTupleDataclassField(
         default=(0.9, 0.999), description="Coefficients used for computing running averages of gradient and its square."
     )
+
     eps: float = NonNegativeFloat(
         default=1e-08, description="Term added to the denominator to improve numerical stability."
     )
+
     weight_decay: float = NonNegativeFloat(default=0.0, description="Weight decay (L2 penalty).")
+
     amsgrad: bool = Boolean(
         default=False,
         description=(
@@ -110,18 +120,22 @@ class AdamWOptimizerConfig(BaseOptimizerConfig):
        (default: 'adamw')"""
 
     # Defaults taken from https://pytorch.org/docs/stable/generated/torch.optim.Adam.html#torch.optim.Adam :
-    lr: float = FloatRange(default=1e-03, min=0.0, max=1.0, description="Learning rate.")
+    lr: float = NonNegativeFloat(default=1e-03, description="Learning rate.")
+
     betas: Tuple[float, float] = FloatRangeTupleDataclassField(
         default=(0.9, 0.999), description="Coefficients used for computing running averages of gradient and its square."
     )
+
     eps: float = NonNegativeFloat(
         default=1e-08, description="Term added to the denominator to improve numerical stability."
     )
+
     weight_decay: float = NonNegativeFloat(default=0.0, description="Weight decay ($L2$ penalty).")
+
     amsgrad: bool = Boolean(
         default=False,
         description=(
-            "Whether to use the AMSGrad variant of this algorithm from the paper 'On the Convergence of Adam and"
+            "Whether to use the AMSGrad variant of this algorithm from the paper 'On the Convergence of Adam and "
             "Beyond'."
         ),
     )
@@ -146,15 +160,16 @@ class AdadeltaOptimizerConfig(BaseOptimizerConfig):
         max=1.0,
         description="Coefficient used for computing a running average of squared gradients.",
     )
+
     eps: float = NonNegativeFloat(
         default=1e-06, description="Term added to the denominator to improve numerical stability."
     )
-    lr: float = FloatRange(
+
+    lr: float = NonNegativeFloat(
         default=1.0,
-        min=0.0,
-        max=1.0,
         description="Coefficient that scales delta before it is applied to the parameters.",
     )
+
     weight_decay: float = NonNegativeFloat(default=0.0, description="Weight decay ($L2$ penalty).")
 
 
@@ -173,9 +188,13 @@ class AdagradOptimizerConfig(BaseOptimizerConfig):
 
     # Defaults taken from https://pytorch.org/docs/stable/generated/torch.optim.Adagrad.html#torch.optim.Adagrad :
     initial_accumulator_value: float = NonNegativeFloat(default=0, description="")
-    lr: float = FloatRange(default=1e-2, min=0.0, max=1.0, description="Learning rate.")
+
+    lr: float = NonNegativeFloat(default=1e-2, description="Learning rate.")
+
     lr_decay: float = FloatRange(default=0, description="Learning rate decay.")
+
     weight_decay: float = FloatRange(default=0, description="Weight decay ($L2$ penalty).")
+
     eps: float = FloatRange(default=1e-10, description="Term added to the denominator to improve numerical stability.")
 
 
@@ -192,13 +211,16 @@ class AdamaxOptimizerConfig(BaseOptimizerConfig):
        (default: 'adamax')"""
 
     # Defaults taken from https://pytorch.org/docs/stable/generated/torch.optim.Adamax.html#torch.optim.Adamax :
-    lr: float = FloatRange(default=2e-3, min=0.0, max=1.0, description="Learning rate.")
+    lr: float = NonNegativeFloat(default=2e-3, description="Learning rate.")
+
     betas: Tuple[float, float] = FloatRangeTupleDataclassField(
         default=(0.9, 0.999), description="Coefficients used for computing running averages of gradient and its square."
     )
+
     eps: float = NonNegativeFloat(
         default=1e-08, description="Term added to the denominator to improve numerical stability."
     )
+
     weight_decay: float = NonNegativeFloat(default=0.0, description="Weight decay ($L2$ penalty).")
 
 
@@ -206,28 +228,42 @@ class AdamaxOptimizerConfig(BaseOptimizerConfig):
 # @register_optimizer(name="ftrl")
 @dataclass
 class FtrlOptimizerConfig(BaseOptimizerConfig):
+
     # optimizer_class: ClassVar[torch.optim.Optimizer] = torch.optim.Ftrl
     type: str = StringOptions(["ftrl"], default="ftrl", allow_none=False)
+
     learning_rate_power: float = FloatRange(default=-0.5, max=0.0)
+
     initial_accumulator_value: float = NonNegativeFloat(default=0.1)
+
     l1_regularization_strength: float = NonNegativeFloat(default=0.0)
+
     l2_regularization_strength: float = NonNegativeFloat(default=0.0)
 
 
-# @register_optimizer(name="nadam")
+@register_optimizer(name="nadam")
 @dataclass
 class NadamOptimizerConfig(BaseOptimizerConfig):
-    # optimizer_class: ClassVar[torch.optim.Optimizer] = torch.optim.Nadam
+
+    optimizer_class: ClassVar[torch.optim.Optimizer] = torch.optim.NAdam
+    """Points to `torch.optim.NAdam`."""
+
     type: str = StringOptions(["nadam"], default="nadam", allow_none=False)
+
     # Defaults taken from https://pytorch.org/docs/stable/generated/torch.optim.NAdam.html#torch.optim.NAdam :
-    lr: float = FloatRange(default=2e-3, min=0.0, max=1.0, description="Learning rate.")
+
+    lr: float = NonNegativeFloat(default=2e-3, description="Learning rate.")
+
     betas: Tuple[float, float] = FloatRangeTupleDataclassField(
         default=(0.9, 0.999), description="Coefficients used for computing running averages of gradient and its square."
     )
+
     eps: float = NonNegativeFloat(
         default=1e-08, description="Term added to the denominator to improve numerical stability."
     )
+
     weight_decay: float = NonNegativeFloat(default=0.0, description="Weight decay ($L2$ penalty).")
+
     momentum_decay: float = NonNegativeFloat(default=4e-3, description="Momentum decay.")
 
 
@@ -244,18 +280,23 @@ class RMSPropOptimizerConfig(BaseOptimizerConfig):
        (default: 'rmsprop')"""
 
     # Defaults taken from https://pytorch.org/docs/stable/generated/torch.optim.RMSprop.html#torch.optim.RMSprop:
-    lr: float = FloatRange(default=1e-2, min=0.0, max=1.0, description="Learning rate.")
+    lr: float = NonNegativeFloat(default=1e-2, description="Learning rate.")
+
     momentum: float = NonNegativeFloat(default=0.0, description="Momentum factor.")
+
     alpha: float = NonNegativeFloat(default=0.99, description="Smoothing constant.")
+
     eps: float = NonNegativeFloat(
         default=1e-08, description="Term added to the denominator to improve numerical stability."
     )
+
     centered: bool = Boolean(
         default=False,
         description=(
             "If True, computes the centered RMSProp, and the gradient is normalized by an estimation of its variance."
         ),
     )
+
     weight_decay: float = NonNegativeFloat(default=0.0, description="Weight decay ($L2$ penalty).")
 
 
@@ -308,7 +349,8 @@ def OptimizerDataclassField(default={"type": "adam"}, description="TODO"):
                 )
             raise ValidationError("Field should be None or dict")
 
-        def _jsonschema_type_mapping(self):
+        @staticmethod
+        def _jsonschema_type_mapping():
             # Note that this uses the same conditional pattern as combiners:
             return {
                 "type": "object",
@@ -325,7 +367,8 @@ def OptimizerDataclassField(default={"type": "adam"}, description="TODO"):
         raise ValidationError(f"Invalid default: `{default}`")
     try:
         opt = optimizer_registry[default["type"].lower()][1]
-        load_default = opt.Schema().load(default)
+        load_default = opt.Schema()
+        load_default = load_default.load(default)
         dump_default = opt.Schema().dump(default)
 
         return field(
@@ -354,13 +397,14 @@ class GradientClippingConfig(BaseMarshmallowConfig):
     clipvalue: Optional[float] = FloatRange(default=None, allow_none=True, description="")
 
 
-def GradientClippingDataclassField(default={}, allow_none=True, description="TODO"):
+def GradientClippingDataclassField(description: str, default: Dict = {}):
     """Returns custom dataclass field for `ludwig.modules.optimization_modules.GradientClippingConfig`. Allows
     `None` by default.
 
     :param default: dict that specifies clipping param values that will be loaded by its schema class (default: {}).
     :param allow_none: Whether this field can accept `None` as a value. (default: True)
     """
+    allow_none = True
 
     class GradientClippingMarshmallowField(fields.Field):
         """Custom marshmallow field class for gradient clipping.
@@ -403,7 +447,10 @@ def GradientClippingDataclassField(default={}, allow_none=True, description="TOD
                 allow_none=allow_none,
                 load_default=load_default,
                 dump_default=dump_default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(TRAINER_METADATA["gradient_clipping"]),
+                },
             )
         },
         default_factory=lambda: load_default,
