@@ -1,25 +1,55 @@
 from dataclasses import field
-from typing import Dict as tDict
-from typing import List, Tuple, Type, Union
+from typing import Any
+from typing import Dict as TDict
+from typing import List as TList
+from typing import Tuple, Type, Union
 
 from marshmallow import EXCLUDE, fields, schema, validate, ValidationError
+from marshmallow_dataclass import dataclass as m_dataclass
 from marshmallow_jsonschema import JSONSchema as js
 
 from ludwig.modules.reduction_modules import reduce_mode_registry
+from ludwig.schema.metadata.parameter_metadata import convert_metadata_to_json, ParameterMetadata
 from ludwig.utils.torch_utils import activations, initializer_registry
 
 
-def load_config(cls: Type["BaseMarshmallowConfig"], **kwargs):  # noqa 0821
+def load_config(cls: Type["BaseMarshmallowConfig"], **kwargs) -> "BaseMarshmallowConfig":  # noqa 0821
     """Takes a marshmallow class and instantiates it with the given keyword args as parameters."""
     assert_is_a_marshmallow_class(cls)
     schema = cls.Schema()
     return schema.load(kwargs)
 
 
+def load_trainer_with_kwargs(
+    model_type: str, kwargs: dict
+) -> Tuple["BaseMarshmallowConfig", TDict[str, Any]]:  # noqa: F821
+    """Special case of `load_config_with_kwargs` for the trainer schemas.
+
+    In particular, it chooses the correct default type for an incoming config (if it doesn't have one already), but
+    otherwise passes all other parameters through without change.
+    """
+    from ludwig.constants import MODEL_ECD, TYPE
+    from ludwig.schema.trainer import ECDTrainerConfig, GBMTrainerConfig
+
+    trainer_schema = ECDTrainerConfig if model_type == MODEL_ECD else GBMTrainerConfig
+
+    def default_type_for_trainer_schema(cls):
+        """Returns the default values for the "type" field on the given trainer schema."""
+        return cls.Schema().fields[TYPE].dump_default
+
+    # Create a copy of kwargs with the correct default type (which will be overridden if kwargs already contains 'type')
+    kwargs_with_type = {**{TYPE: default_type_for_trainer_schema(trainer_schema)}, **kwargs}
+
+    return load_config_with_kwargs(trainer_schema, kwargs_with_type)
+
+
 def load_config_with_kwargs(
     cls: Type["BaseMarshmallowConfig"], kwargs_overrides
-) -> "BaseMarshmallowConfig":  # noqa 0821
-    """Instatiates an instance of the marshmallow class and kwargs overrides instantiantes the schema."""
+) -> Tuple["BaseMarshmallowConfig", TDict[str, Any]]:  # noqa 0821
+    """Instatiates an instance of the marshmallow class and kwargs overrides instantiantes the schema.
+
+    Returns a tuple of config, and a dictionary of any keys in kwargs_overrides which are no present in config.
+    """
     assert_is_a_marshmallow_class(cls)
     schema = cls.Schema()
     fields = schema.fields.keys()
@@ -28,7 +58,7 @@ def load_config_with_kwargs(
     }
 
 
-def create_cond(if_pred: tDict, then_pred: tDict):
+def create_cond(if_pred: TDict, then_pred: TDict):
     """Returns a JSONSchema conditional for the given if-then predicates."""
     return {
         "if": {"properties": {k: {"const": v} for k, v in if_pred.items()}},
@@ -52,6 +82,13 @@ class BaseMarshmallowConfig:
         unknown = EXCLUDE
         "Flag that sets marshmallow `load` calls to ignore unknown properties passed as a parameter."
 
+    def to_dict(self):
+        """Method for getting a dictionary representation of this dataclass.
+
+        Returns: dict for this dataclass
+        """
+        return self.__dict__
+
 
 def assert_is_a_marshmallow_class(cls):
     assert hasattr(cls, "Schema") and isinstance(
@@ -59,7 +96,7 @@ def assert_is_a_marshmallow_class(cls):
     ), f"Expected marshmallow class, but `{cls}` does not have the necessary `Schema` attribute."
 
 
-def unload_jsonschema_from_marshmallow_class(mclass) -> tDict:
+def unload_jsonschema_from_marshmallow_class(mclass) -> TDict:
     """Helper method to directly get a marshmallow class's JSON schema without extra wrapping props."""
     assert_is_a_marshmallow_class(mclass)
     schema = js().dump(mclass.Schema())["definitions"][mclass.__name__]
@@ -72,8 +109,8 @@ def InitializerOptions(default: str = "xavier_uniform", description=""):
     return StringOptions(list(initializer_registry.keys()), default=default, allow_none=False, description=description)
 
 
-def ActivationOptions(default: str = "relu", description=""):
-    """Utility warapper that returns a `StringOptions` field with keys from `activations` registry."""
+def ActivationOptions(default: Union[str, None] = "relu", description=""):
+    """Utility wrapper that returns a `StringOptions` field with keys from `activations` registry."""
     return StringOptions(list(activations.keys()), default=default, allow_none=True, description=description)
 
 
@@ -87,20 +124,43 @@ def RegularizerOptions(default: Union[None, str] = None, allow_none: bool = True
     return StringOptions(["l1", "l2", "l1_l2"], default=default, allow_none=allow_none, description=description)
 
 
-def String(default: Union[None, str] = None, allow_none: bool = True, description=""):
+def String(
+    description: str,
+    default: Union[None, str] = None,
+    allow_none: bool = True,
+    pattern: str = None,
+    parameter_metadata: ParameterMetadata = None,
+):
     if not allow_none and not isinstance(default, str):
         raise ValidationError(f"Provided default `{default}` should be a string!")
+
+    if pattern is not None:
+        validation = validate.Regexp(pattern)
+    else:
+        validation = None
+
     return field(
         metadata={
             "marshmallow_field": fields.String(
-                allow_none=allow_none, load_default=default, dump_default=default, metadata={"description": description}
-            )
+                validate=validation,
+                allow_none=allow_none,
+                load_default=default,
+                dump_default=default,
+                metadata={"description": description},
+            ),
+            "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
         },
         default=default,
     )
 
 
-def StringOptions(options: List[str], default: Union[None, str] = None, allow_none: bool = True, description=""):
+def StringOptions(
+    options: TList[str],
+    default: Union[None, str] = None,
+    allow_none: bool = True,
+    description: str = "",
+    parameter_metadata: ParameterMetadata = None,
+):
     """Returns a dataclass field with marshmallow metadata that enforces string inputs must be one of `options`.
 
     By default, None is allowed (and automatically appended) to the allowed list of options.
@@ -124,14 +184,17 @@ def StringOptions(options: List[str], default: Union[None, str] = None, allow_no
                 allow_none=allow_none,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default=default,
     )
 
 
-def Boolean(default: bool, description=""):
+def Boolean(default: bool, description: str, parameter_metadata: ParameterMetadata = None):
     if default is not None:
         try:
             assert isinstance(default, bool)
@@ -145,14 +208,47 @@ def Boolean(default: bool, description=""):
                 allow_none=False,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default=default,
     )
 
 
-def PositiveInteger(default: Union[None, int] = None, allow_none=False, description=""):
+def Integer(
+    default: Union[None, int] = None, allow_none=False, description="", parameter_metadata: ParameterMetadata = None
+):
+    """Returns a dataclass field with marshmallow metadata strictly enforcing (non-float) inputs."""
+    allow_none = allow_none or default is None
+
+    if default is not None:
+        try:
+            assert isinstance(default, int)
+        except Exception:
+            raise ValidationError(f"Invalid default: `{default}`")
+    return field(
+        metadata={
+            "marshmallow_field": fields.Integer(
+                strict=True,
+                allow_none=allow_none,
+                load_default=default,
+                dump_default=default,
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
+            )
+        },
+        default=default,
+    )
+
+
+def PositiveInteger(
+    description: str, default: Union[None, int], allow_none: bool = False, parameter_metadata: ParameterMetadata = None
+):
     """Returns a dataclass field with marshmallow metadata strictly enforcing (non-float) inputs must be
     positive."""
     val = validate.Range(min=1)
@@ -172,14 +268,22 @@ def PositiveInteger(default: Union[None, int] = None, allow_none=False, descript
                 allow_none=allow_none,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default=default,
     )
 
 
-def NonNegativeInteger(default: Union[None, int] = None, allow_none=False, description=""):
+def NonNegativeInteger(
+    description: str,
+    default: Union[None, int] = None,
+    allow_none: bool = False,
+    parameter_metadata: ParameterMetadata = None,
+):
     """Returns a dataclass field with marshmallow metadata strictly enforcing (non-float) inputs must be
     nonnegative."""
     val = validate.Range(min=0)
@@ -199,17 +303,29 @@ def NonNegativeInteger(default: Union[None, int] = None, allow_none=False, descr
                 allow_none=allow_none,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default=default,
     )
 
 
-def IntegerRange(default: Union[None, int] = None, allow_none=False, description="", **kwargs):
+def IntegerRange(
+    description: str,
+    default: Union[None, int] = None,
+    allow_none=False,
+    parameter_metadata: ParameterMetadata = None,
+    min: int = None,
+    max: int = None,
+    min_inclusive: bool = True,
+    max_inclusive: bool = True,
+):
     """Returns a dataclass field with marshmallow metadata strictly enforcing (non-float) inputs must be in range
     set by relevant keyword args."""
-    val = validate.Range(**kwargs)
+    val = validate.Range(min=min, max=max, min_inclusive=min_inclusive, max_inclusive=max_inclusive)
     allow_none = allow_none or default is None
 
     if default is not None:
@@ -226,14 +342,22 @@ def IntegerRange(default: Union[None, int] = None, allow_none=False, description
                 allow_none=allow_none,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default=default,
     )
 
 
-def NonNegativeFloat(default: Union[None, float] = None, allow_none=False, description=""):
+def NonNegativeFloat(
+    default: Union[None, float] = None,
+    allow_none=False,
+    description: str = "",
+    parameter_metadata: ParameterMetadata = None,
+):
     """Returns a dataclass field with marshmallow metadata enforcing numeric inputs must be nonnegative."""
     val = validate.Range(min=0.0)
     allow_none = allow_none or default is None
@@ -251,17 +375,29 @@ def NonNegativeFloat(default: Union[None, float] = None, allow_none=False, descr
                 allow_none=allow_none,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default=default,
     )
 
 
-def FloatRange(default: Union[None, float] = None, allow_none=False, description="", **kwargs):
+def FloatRange(
+    default: Union[None, float] = None,
+    allow_none: bool = False,
+    description: str = "",
+    parameter_metadata: ParameterMetadata = None,
+    min: int = None,
+    max: int = None,
+    min_inclusive: bool = True,
+    max_inclusive: bool = True,
+):
     """Returns a dataclass field with marshmallow metadata enforcing numeric inputs must be in range set by
     relevant keyword args."""
-    val = validate.Range(**kwargs)
+    val = validate.Range(min=min, max=max, min_inclusive=min_inclusive, max_inclusive=max_inclusive)
     allow_none = allow_none or default is None
 
     if default is not None:
@@ -277,14 +413,154 @@ def FloatRange(default: Union[None, float] = None, allow_none=False, description
                 allow_none=allow_none,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default=default,
     )
 
 
-def Dict(default: Union[None, tDict] = None, description=""):
+def IntegerOrSequenceOfIntegers(
+    default: Union[None, int, Tuple[int, ...], TList[int]] = None,
+    default_integer: int = None,
+    default_sequence: Union[TList[int], Tuple[int, ...]] = None,
+    allow_none=False,
+    non_negative: bool = True,
+    description="",
+):
+    """Returns a dataclass field with marshmallow metadata enforcing numeric inputs or a tuple of numeric
+    inputs."""
+
+    class IntegerOrIntegerSequenceField(fields.Field):
+        def _deserialize(self, value, attr, data, **kwargs):
+            if isinstance(value, int):
+                if non_negative:
+                    if value < 0:
+                        raise ValidationError("Value must be positive.")
+                return value
+            if isinstance(value, (tuple, list)):
+                if non_negative:
+                    for v in value:
+                        if v < 0:
+                            raise ValidationError("Values must be positive.")
+                return value
+            raise ValidationError("Field should be either an integer, tuple of integers, or a list of integers")
+
+        def _jsonschema_type_mapping(self):
+            numeric_option = {
+                "type": "integer",
+                "title": "integer_option",
+                "default": default_integer,
+                "description": "Set to a valid number.",
+            }
+            sequence_option = {
+                "type": "array",
+                "title": "sequence_option",
+                "items": {"type": "number"},
+                "default": default_sequence,
+                "description": "Set to a valid number.",
+            }
+
+            oneof_list = [
+                numeric_option,
+                sequence_option,
+            ]
+
+            return {"oneOf": oneof_list, "title": self.name, "description": description, "default": default}
+
+    return field(
+        metadata={
+            "marshmallow_field": IntegerOrIntegerSequenceField(
+                allow_none=allow_none, load_default=default, dump_default=default, metadata={"description": description}
+            )
+        },
+        default=default,
+    )
+
+
+def PositiveIntegerOrTupleOrStringOptions(
+    options: TList[str] = None,
+    allow_none=False,
+    default: Union[None, int, Tuple[int, ...], str] = None,
+    default_integer: Union[None, int] = None,
+    default_tuple: Union[None, Tuple[int, ...]] = None,
+    default_option: Union[None, str] = None,
+    description="",
+):
+    """Returns a dataclass field with marshmallow metadata enforcing numeric inputs, a tuple of numeric inputs, or
+    a string value."""
+
+    class IntegerTupleStringOptionsField(fields.Field):
+        def _deserialize(self, value, attr, data, **kwargs):
+            if isinstance(value, int):
+                if value < 0:
+                    raise ValidationError("Value must be positive.")
+                return value
+            if isinstance(value, tuple):
+                for v in value:
+                    if v < 0:
+                        raise ValidationError("Values must be positive.")
+                return value
+            if isinstance(value, str):
+                if value not in options:
+                    raise ValidationError(f"String value should be one of {options}")
+                return value
+
+            raise ValidationError("Field should be either an integer, tuple of integers, or a string")
+
+        def _jsonschema_type_mapping(self):
+            if None in options and not self.allow_none:
+                raise AssertionError(
+                    f"Provided string options `{options}` includes `None`, but field is not set to allow `None`."
+                )
+
+            # Prepare numeric option:
+            numeric_option = {
+                "type": "integer",
+                "title": "integer_option",
+                "default": default_integer,
+                "description": "Set to a valid number.",
+            }
+            tuple_option = {
+                "type": "array",
+                "title": "tuple_option",
+                "items": [{"type": "number", "minimum": 0, "maximum": 999999}] * 2,
+                "default": default_tuple,
+                "description": "Set to a valid number.",
+            }
+
+            # Prepare string option (remove None):
+            if None in options:
+                options.remove(None)
+            string_option = {
+                "type": "string",
+                "enum": options,
+                "default": default_option,
+                "title": "preconfigured_option",
+                "description": "Choose a preconfigured option.",
+            }
+            oneof_list = [
+                numeric_option,
+                tuple_option,
+                string_option,
+            ]
+
+            return {"oneOf": oneof_list, "title": self.name, "description": description, "default": default}
+
+    return field(
+        metadata={
+            "marshmallow_field": IntegerTupleStringOptionsField(
+                allow_none=allow_none, load_default=default, dump_default=default, metadata={"description": description}
+            )
+        },
+        default=default,
+    )
+
+
+def Dict(default: Union[None, TDict] = None, description: str = "", parameter_metadata: ParameterMetadata = None):
     """Returns a dataclass field with marshmallow metadata enforcing input must be a dict."""
     if default is not None:
         try:
@@ -299,6 +575,43 @@ def Dict(default: Union[None, tDict] = None, description=""):
                 allow_none=True,
                 load_default=default,
                 dump_default=default,
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
+            )
+        },
+        default_factory=lambda: default,
+    )
+
+
+def List(
+    list_type: Union[Type[str], Type[int], Type[float]] = str, default: Union[None, TList[Any]] = None, description=""
+):
+    """Returns a dataclass field with marshmallow metadata enforcing input must be a list."""
+    if default is not None:
+        try:
+            assert isinstance(default, list)
+
+        except Exception:
+            raise ValidationError(f"Invalid default: `{default}`")
+
+    if list_type is str:
+        field_type = fields.String()
+    elif list_type is int:
+        field_type = fields.Integer()
+    elif list_type is float:
+        field_type = fields.Float()
+    else:
+        raise ValueError(f"Invalid list type: `{list_type}`")
+
+    return field(
+        metadata={
+            "marshmallow_field": fields.List(
+                field_type,
+                allow_none=True,
+                load_default=default,
+                dump_default=default,
                 metadata={"description": description},
             )
         },
@@ -306,7 +619,9 @@ def Dict(default: Union[None, tDict] = None, description=""):
     )
 
 
-def DictList(default: Union[None, List[tDict]] = None, description=""):
+def DictList(
+    default: Union[None, TList[TDict]] = None, description: str = "", parameter_metadata: ParameterMetadata = None
+):
     """Returns a dataclass field with marshmallow metadata enforcing input must be a list of dicts."""
     if default is not None:
         try:
@@ -324,7 +639,10 @@ def DictList(default: Union[None, List[tDict]] = None, description=""):
                 allow_none=True,
                 load_default=default,
                 dump_default=default,
-                metadata={"description": description},
+                metadata={
+                    "description": description,
+                    "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
+                },
             )
         },
         default_factory=lambda: default,
@@ -380,7 +698,7 @@ def Embed():
     )
 
 
-def InitializerOrDict(default: str = "xavier_uniform", description=""):
+def InitializerOrDict(default: str = "xavier_uniform", description: str = ""):
     """Returns a dataclass field with marshmallow metadata allowing customizable initializers.
 
     In particular, allows str or dict types; in the former case the field is equivalent to `InitializerOptions` while in
@@ -411,13 +729,6 @@ def InitializerOrDict(default: str = "xavier_uniform", description=""):
             initializers = list(initializer_registry.keys())
             return {
                 "oneOf": [
-                    {
-                        "type": "string",
-                        "enum": initializers,
-                        "default": default,
-                        "title": "initializer_preconfigured_option",
-                        "description": "Pick a preconfigured initializer.",
-                    },
                     # Note: default not provided in the custom dict option:
                     {
                         "type": "object",
@@ -429,6 +740,13 @@ def InitializerOrDict(default: str = "xavier_uniform", description=""):
                         "additionalProperties": True,
                         "description": "Customize an existing initializer.",
                     },
+                    {
+                        "type": "string",
+                        "enum": initializers,
+                        "default": default,
+                        "title": "initializer_preconfigured_option",
+                        "description": "Pick a preconfigured initializer.",
+                    },
                 ],
                 "title": self.name,
                 "default": default,
@@ -445,31 +763,42 @@ def InitializerOrDict(default: str = "xavier_uniform", description=""):
     )
 
 
-def FloatRangeTupleDataclassField(N=2, default: Tuple = (0.9, 0.999), min=0, max=1, description=""):
-    """Returns a dataclass field with marshmallow metadata enforcing a `N`-dim. tuple with all values in given
-    range.
+def FloatRangeTupleDataclassField(
+    n=2, default: Union[Tuple, None] = (0.9, 0.999), allow_none: bool = False, min=0, max=1, description=""
+):
+    """Returns a dataclass field with marshmallow metadata enforcing a `N`-dim.
 
-    In particular, inputs must be N-dimensional tuples of purely numeric values within [min, max] range, i.e. inclusive.
-    The generated JSON schema uses a restricted array type as the equivalent representation of a Python tuple.
+    tuple with all values in given range. In particular, inputs must be N-dimensional tuples of purely numeric values
+    within [min, max] range, i.e. inclusive. The generated JSON schema uses a restricted array type as the equivalent
+    representation of a Python tuple.
     """
-    if N != len(default):
-        raise ValidationError(f"Dimension of tuple '{N}' must match dimension of default val. '{default}'")
+    if default is not None and n != len(default):
+        raise ValidationError(f"Dimension of tuple '{n}' must match dimension of default val. '{default}'")
 
     class FloatTupleMarshmallowField(fields.Tuple):
         def _jsonschema_type_mapping(self):
-            validate_range(default)
+            if default is not None:
+                validate_range(default)
             return {
-                "type": "array",
-                "items": [
+                "oneOf": [
                     {
-                        "type": "number",
-                        "minimum": min,
-                        "maximum": max,
-                    }
-                ]
-                * N,
+                        "type": "array",
+                        "items": [
+                            {
+                                "type": "number",
+                                "minimum": min,
+                                "maximum": max,
+                            }
+                        ]
+                        * n,
+                        "default": default,
+                        "description": description,
+                    },
+                    {"type": "null", "title": "null_float_tuple_option", "description": "None"},
+                ],
+                "title": self.name,
                 "default": default,
-                "description": description,
+                "description": "Valid options for FloatRangeTupleDataclassField.",
             }
 
     def validate_range(data: Tuple):
@@ -479,18 +808,21 @@ def FloatRangeTupleDataclassField(N=2, default: Tuple = (0.9, 0.999), min=0, max
             raise ValidationError(
                 f"Values in received tuple should be in range [{min},{max}], instead received: {data}"
             )
-        raise ValidationError(f'Received value should be of {N}-dimensional "Tuple[float]", instead received: {data}')
+        raise ValidationError(f'Received value should be of {n}-dimensional "Tuple[float]", instead received: {data}')
 
     try:
-        validate_range(default)
+        if default is not None:
+            validate_range(default)
+        if default is None and not allow_none:
+            raise ValidationError("Default value must not be None if allow_none is False")
     except Exception:
         raise ValidationError(f"Invalid default: `{default}`")
 
     return field(
         metadata={
             "marshmallow_field": FloatTupleMarshmallowField(
-                tuple_fields=[fields.Float()] * N,
-                allow_none=False,
+                tuple_fields=[fields.Float()] * n,
+                allow_none=allow_none,
                 validate=validate_range,
                 load_default=default,
                 dump_default=default,
@@ -501,123 +833,92 @@ def FloatRangeTupleDataclassField(N=2, default: Tuple = (0.9, 0.999), min=0, max
     )
 
 
-def IntegerOrStringOptionsField(
-    options: List[str],
-    allow_none: bool,
-    default: Union[None, int],
-    default_numeric: Union[None, int],
-    default_option: Union[None, str],
-    is_integer: bool = True,
-    min: Union[None, int] = None,
-    max: Union[None, int] = None,
-    min_exclusive: Union[None, int] = None,
-    max_exclusive: Union[None, int] = None,
-    description="",
+def OneOfOptionsField(
+    default: Any, description: str, allow_none: bool, field_options: TList, parameter_metadata: ParameterMetadata = None
 ):
-    """Returns a dataclass field with marshmallow metadata enforcing strict integers or protected strings."""
-    is_integer = True
-    return NumericOrStringOptionsField(**locals())
+    """Returns a dataclass field that is a combination of the other fields defined in `ludwig.schema.utils`."""
+    field_options_allow_none = any(option.metadata["marshmallow_field"].allow_none for option in field_options)
 
+    class OneOfOptionsCombinatorialField(fields.Field):
+        def _serialize(self, value, attr, obj, **kwargs):
+            if allow_none and value is None:
+                return None
+            for option in field_options:
+                mfield_meta = option.metadata["marshmallow_field"]
+                try:
+                    if value is None and mfield_meta.allow_none:
+                        return None
+                    mfield_meta.validate(value)
+                    return mfield_meta._serialize(value, attr, obj, **kwargs)
+                except Exception:
+                    continue
+            raise ValidationError(f"Value to serialize does not match any valid option schemas: {value}")
 
-def NumericOrStringOptionsField(
-    options: List[str],
-    allow_none: bool,
-    default: Union[None, int, float, str],
-    default_numeric: Union[None, int, float],
-    default_option: Union[None, str],
-    is_integer: bool = False,
-    min: Union[None, int] = None,
-    max: Union[None, int] = None,
-    min_exclusive: Union[None, int] = None,
-    max_exclusive: Union[None, int] = None,
-    description="",
-):
-    """Returns a dataclass field with marshmallow metadata enforcing numeric values or protected strings.
-
-    In particular, numeric values can be constrained to a range through the other arguments, both inclusive and
-    exclusive. Strings must conform to the given set of options (and None/null must be set to be allowed or not).
-    """
-
-    class IntegerOrStringOptionsField(fields.Field):
-        def _deserialize(self, value, attr, data, **kwargs):
-            msg_type = "integer" if is_integer else "numeric"
-            if (is_integer and isinstance(value, int)) or isinstance(value, float):
-                if (
-                    (min is not None and value < min)
-                    or (min_exclusive is not None and value <= min_exclusive)
-                    or (max is not None and value > max)
-                    or (max_exclusive is not None and value >= max_exclusive)
-                ):
-                    err_min_r, err_min_n = "(", min_exclusive if min_exclusive is not None else "[", min
-                    errMaxR, errMaxN = ")", max_exclusive if max_exclusive is not None else "]", max
-                    raise ValidationError(
-                        f"If value is {msg_type} should be in range: {err_min_r}{err_min_n},{errMaxN}{errMaxR}"
-                    )
-                return value
-            if isinstance(value, str):
-                if value not in options:
-                    raise ValidationError(f"String value should be one of {options}")
-                return value
-
-            raise ValidationError(f"Field should be either a {msg_type} or string")
+        def _deserialize(self, value, attr, obj, **kwargs):
+            if allow_none and value is None:
+                return None
+            for option in field_options:
+                mfield_meta = option.metadata["marshmallow_field"]
+                try:
+                    if mfield_meta.validate:
+                        mfield_meta.validate(value)
+                    return mfield_meta._deserialize(value, attr, obj, **kwargs)
+                except Exception:
+                    continue
+            raise ValidationError(f"Value to deserialize does not match any valid option schemas: {value}")
 
         def _jsonschema_type_mapping(self):
-            # Note: schemas can normally support a list of enums that includes 'None' as an option, as we currently have
-            # in 'initializers_registry'. But to make the schema here a bit more straightforward, the user must
-            # explicitly state if 'None' is going to be supported; if this conflicts with the list of enums then an
-            # error is raised and if it's going to be supported then it will be as a separate subschema rather than as
-            # part of the string subschema (see below):
-            if None in options and not self.allow_none:
-                raise AssertionError(
-                    f"Provided string options `{options}` includes `None`, but field is not set to allow `None`."
-                )
+            """Constructs a oneOf schema by iteratively adding the schemas of `field_options` to a list."""
+            oneOf = {"oneOf": [], "description": description, "default": default, "title": self.name}
 
-            # Prepare numeric option:
-            numeric_type = "integer" if is_integer else "number"
-            numeric_option = {
-                "type": numeric_type,
-                "title": numeric_type + "_option",
-                "default": default_numeric,
-                "description": "Set to a valid number.",
-            }
-            if min is not None:
-                numeric_option["minimum"] = min
-            if min_exclusive is not None:
-                numeric_option["exclusiveMinimum"] = min_exclusive
-            if max is not None:
-                numeric_option["maximum"] = max
-            if max_exclusive is not None:
-                numeric_option["exclusiveMaximum"] = max_exclusive
+            for idx, option in enumerate(field_options):
+                mfield_meta = option.metadata["marshmallow_field"]
 
-            # Prepare string option (remove None):
-            if None in options:
-                options.remove(None)
-            string_option = {
-                "type": "string",
-                "enum": options,
-                "default": default_option,
-                "title": "preconfigured_option",
-                "description": "Choose a preconfigured option.",
-            }
-            oneof_list = [
-                numeric_option,
-                string_option,
-            ]
+                # Necessary for key/name de-duplication in case a name is not supplied by the user:
+                mfield_meta_class_name = str(mfield_meta.__class__).split(".")[-1].split("'")[0].lower()
 
-            # Add null as an option if applicable:
-            oneof_list += (
+                # If the option inherits from a custom dataclass-field, then use the custom jsonschema:
+                if hasattr(mfield_meta, "_jsonschema_type_mapping"):
+                    oneOf["oneOf"].append(mfield_meta._jsonschema_type_mapping())
+                # Otherwise, extract the jsonschema using a dummy dataclass as intermediary:
+                else:
+
+                    @m_dataclass
+                    class DummyClass:
+                        tmp: Any = option
+
+                    dummy_schema = unload_jsonschema_from_marshmallow_class(DummyClass)
+                    tmp_json_schema = dummy_schema["properties"]["tmp"]
+                    # Manually set the title, otherwise it would be 'tmp':
+                    tmp_json_schema["title"] = f"{self.name}_{mfield_meta_class_name}_option"
+                    oneOf["oneOf"].append(tmp_json_schema)
+
+            # Add null as an option if none of the field options allow none:
+            oneOf["oneOf"] += (
                 [{"type": "null", "title": "null_option", "description": "Disable this parameter."}]
-                if allow_none
+                if allow_none and not field_options_allow_none
                 else []
             )
 
-            return {"oneOf": oneof_list, "title": self.name, "description": description, "default": default}
+            return oneOf
+
+    # Create correct default kwarg to pass to dataclass field constructor:
+    def is_primitive(value):
+        primitive = (int, str, bool)
+        return isinstance(value, primitive)
+
+    default_kwarg = {}
+    if is_primitive(default):
+        default_kwarg["default"] = default
+    else:
+        default_kwarg["default_factory"] = lambda: default
 
     return field(
         metadata={
-            "marshmallow_field": IntegerOrStringOptionsField(
+            "marshmallow_field": OneOfOptionsCombinatorialField(
                 allow_none=allow_none, load_default=default, dump_default=default, metadata={"description": description}
-            )
+            ),
+            "parameter_metadata": convert_metadata_to_json(parameter_metadata) if parameter_metadata else None,
         },
-        default=default,
+        **default_kwarg,
     )
