@@ -14,8 +14,10 @@
 # limitations under the License.
 # ==============================================================================
 import copy
+import functools
 import os
 import random
+import weakref
 from collections import OrderedDict
 from collections.abc import Mapping
 
@@ -23,6 +25,7 @@ import numpy
 import torch
 
 from ludwig.constants import PROC_COLUMN
+from ludwig.globals import DESCRIPTION_FILE_NAME
 from ludwig.utils.fs_utils import find_non_existing_dir_by_adding_suffix
 
 
@@ -96,10 +99,24 @@ def set_default_value(dictionary, key, value):
         dictionary[key] = value
 
 
-def set_default_values(dictionary, default_value_dictionary):
-    # Set multiple default values
+def set_default_values(dictionary: dict, default_value_dictionary: dict):
+    """This function sets multiple default values recursively for various areas of the config. By using the helper
+    function set_default_value, It parses input values that contain nested dictionaries, only setting values for
+    parameters that have not already been defined by the user.
+
+    Args:
+        dictionary (dict): The dictionary to set default values for, generally a section of the config.
+        default_value_dictionary (dict): The dictionary containing the default values for the config.
+    """
     for key, value in default_value_dictionary.items():
-        set_default_value(dictionary, key, value)
+        if key not in dictionary:  # Event where the key is not in the dictionary yet
+            dictionary[key] = value
+        elif value == {}:  # Event where dict is empty
+            set_default_value(dictionary, key, value)
+        elif isinstance(value, dict) and value:  # Event where dictionary is nested - recursive call
+            set_default_values(dictionary[key], value)
+        else:
+            set_default_value(dictionary, key, value)
 
 
 def get_class_attributes(c):
@@ -112,7 +129,7 @@ def get_output_directory(output_directory, experiment_name, model_name="run"):
 
 
 def get_file_names(output_directory):
-    description_fn = os.path.join(output_directory, "description.json")
+    description_fn = os.path.join(output_directory, DESCRIPTION_FILE_NAME)
     training_stats_fn = os.path.join(output_directory, "training_statistics.json")
 
     model_dir = os.path.join(output_directory, "model")
@@ -140,3 +157,30 @@ def set_saved_weights_in_checkpoint_flag(config):
     """
     for input_feature in config.get("input_features", []):
         input_feature["saved_weights_in_checkpoint"] = True
+
+
+def remove_empty_lines(str):
+    return "\n".join([line.rstrip() for line in str.split("\n") if line.rstrip()])
+
+
+# TODO(travis): move to cached_property when we drop Python 3.7.
+# https://stackoverflow.com/a/33672499
+def memoized_method(*lru_args, **lru_kwargs):
+    def decorator(func):
+        @functools.wraps(func)
+        def wrapped_func(self, *args, **kwargs):
+            # We're storing the wrapped method inside the instance. If we had
+            # a strong reference to self the instance would never die.
+            self_weak = weakref.ref(self)
+
+            @functools.wraps(func)
+            @functools.lru_cache(*lru_args, **lru_kwargs)
+            def cached_method(*args, **kwargs):
+                return func(self_weak(), *args, **kwargs)
+
+            setattr(self, func.__name__, cached_method)
+            return cached_method(*args, **kwargs)
+
+        return wrapped_func
+
+    return decorator
