@@ -42,6 +42,8 @@ START_SYMBOL = "<SOS>"
 PADDING_SYMBOL = "<PAD>"
 UNKNOWN_SYMBOL = "<UNK>"
 
+logger = logging.getLogger(__name__)
+
 
 class SpecialSymbol(Enum):
     """Special symbols used for text features."""
@@ -88,7 +90,9 @@ def str2bool(v: str, fallback_true_label=None) -> bool:
     if v_str in BOOL_FALSE_STRS:
         return False
     if fallback_true_label is None:
-        raise ValueError(f"Cannot automatically map value {v} to a boolean and no `fallback_true_label` specified")
+        raise ValueError(
+            f"Cannot automatically map value '{v}' to a boolean and no `preprocessing.fallback_true_label` specified"
+        )
     return v == fallback_true_label
 
 
@@ -258,7 +262,7 @@ def create_vocabulary(
     # Pre-trained huggingface tokenizer. Use the pre-existing vocabulary and special symbols.
     if tokenizer_type == "hf_tokenizer":
         try:
-            vocab = tokenizer.tokenizer.get_vocab()
+            vocab = tokenizer.get_vocab()
             vocab = list(vocab.keys())
         except NotImplementedError:
             vocab = []
@@ -266,8 +270,8 @@ def create_vocabulary(
                 vocab.append(tokenizer.tokenizer._convert_id_to_token(idx))
             vocab += tokenizer.tokenizer.added_tokens_encoder.keys()
 
-        pad_token = tokenizer.tokenizer.pad_token
-        unk_token = tokenizer.tokenizer.unk_token
+        pad_token = tokenizer.get_pad_token()
+        unk_token = tokenizer.get_unk_token()
 
         if unk_token is None:
             vocab = [unknown_symbol] + vocab
@@ -280,6 +284,7 @@ def create_vocabulary(
             padding_symbol = pad_token
     elif hasattr(tokenizer, "get_vocab"):
         vocab = tokenizer.get_vocab()
+        vocab = list(vocab.keys())
     elif vocab_file is not None:
         vocab = load_vocabulary(vocab_file)
 
@@ -327,9 +332,6 @@ def create_vocabulary_single_token(
     This assumption allows us to be more efficient than `create_vocabulary()` as we can skip tokenization and
     computing the maximum sequence length, which are unnecessary for category features.
 
-    The `UNKNOWN` special symbol is always included in the final vocabulary. Additional special symbols (PADDING, START,
-    STOP) are added if add_special_symbols=True.
-
     Args:
         data: Series of string data.
         num_most_frequent: Upper limit on vocabulary size.
@@ -344,7 +346,12 @@ def create_vocabulary_single_token(
     """
     processed_counts = data.str.strip().value_counts(sort=True)
     processed_counts = processor.compute(processed_counts)
-    vocab = [unknown_symbol] + processed_counts.index.tolist()[:num_most_frequent]
+    full_vocab = processed_counts.index.tolist()
+    # Only add unknown symbol if num most frequent tokens is less than total number of unique tokens
+    if num_most_frequent < len(full_vocab):
+        vocab = [unknown_symbol] + full_vocab[:num_most_frequent]
+    else:
+        vocab = full_vocab
     str2idx = {unit: i for i, unit in enumerate(vocab)}
     str2freq = processed_counts.to_dict()
     str2freq = {k: str2freq.get(k, 0) for k in vocab}
@@ -410,7 +417,7 @@ def build_sequence_matrix(
 
     max_length = processor.compute(unit_vectors.map(len).max())
     if max_length < length_limit:
-        logging.debug(f"max length of {format}: {max_length} < limit: {length_limit}")
+        logger.debug(f"max length of {format}: {max_length} < limit: {length_limit}")
     max_length = length_limit
 
     def pad(vector):
