@@ -25,7 +25,7 @@ import yaml
 
 from ludwig.api import LudwigModel
 from ludwig.backend import LOCAL_BACKEND
-from ludwig.constants import H3, TRAINER
+from ludwig.constants import ENCODER, H3, TRAINER, TYPE
 from ludwig.data.concatenate_datasets import concatenate_df
 from ludwig.data.preprocessing import preprocess_for_training
 from ludwig.encoders.registry import get_encoder_classes
@@ -66,8 +66,10 @@ logging.getLogger("ludwig").setLevel(logging.INFO)
 
 @pytest.mark.parametrize("encoder", ENCODERS)
 def test_experiment_text_feature_non_HF(encoder, csv_filename):
-    input_features = [text_feature(vocab_size=30, min_len=1, encoder=encoder, preprocessing={"tokenizer": "space"})]
-    output_features = [category_feature(vocab_size=2)]
+    input_features = [
+        text_feature(encoder={"vocab_size": 30, "min_len": 1, "type": encoder}, preprocessing={"tokenizer": "space"})
+    ]
+    output_features = [category_feature(decoder={"vocab_size": 2})]
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -76,14 +78,8 @@ def test_experiment_text_feature_non_HF(encoder, csv_filename):
 def run_experiment_with_encoder(encoder, csv_filename):
     # Run in a subprocess to clear TF and prevent OOM
     # This also allows us to use GPU resources
-    input_features = [
-        text_feature(
-            vocab_size=30,
-            min_len=1,
-            encoder=encoder,
-        )
-    ]
-    output_features = [category_feature(vocab_size=2)]
+    input_features = [text_feature(encoder={"vocab_size": 30, "min_len": 1, "type": encoder})]
+    output_features = [category_feature(decoder={"vocab_size": 2})]
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -102,8 +98,8 @@ def test_experiment_text_feature_HF_full(encoder, csv_filename):
 
 @pytest.mark.parametrize("encoder", ENCODERS)
 def test_experiment_seq_seq_generator(csv_filename, encoder):
-    input_features = [text_feature(reduce_output=None, encoder=encoder)]
-    output_features = [text_feature(decoder="generator")]
+    input_features = [text_feature(encoder={"type": encoder, "reduce_output": None})]
+    output_features = [text_feature(decoder={"type": "generator"})]
     rel_path = generate_data(input_features, output_features, csv_filename)
 
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -111,8 +107,8 @@ def test_experiment_seq_seq_generator(csv_filename, encoder):
 
 @pytest.mark.parametrize("encoder", ["embed", "rnn", "parallel_cnn", "stacked_parallel_cnn", "transformer"])
 def test_experiment_seq_seq_tagger(csv_filename, encoder):
-    input_features = [text_feature(reduce_output=None, encoder=encoder)]
-    output_features = [text_feature(decoder="tagger")]
+    input_features = [text_feature(encoder={"type": encoder, "reduce_output": None})]
+    output_features = [text_feature(decoder={"type": "tagger"})]
     rel_path = generate_data(input_features, output_features, csv_filename)
 
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -120,8 +116,8 @@ def test_experiment_seq_seq_tagger(csv_filename, encoder):
 
 @pytest.mark.parametrize("encoder", ["cnnrnn", "stacked_cnn"])
 def test_experiment_seq_seq_tagger_fails_for_non_length_preserving_encoders(csv_filename, encoder):
-    input_features = [text_feature(reduce_output=None, encoder=encoder)]
-    output_features = [text_feature(decoder="tagger")]
+    input_features = [text_feature(encoder={"type": encoder, "reduce_output": None})]
+    output_features = [text_feature(decoder={"type": "tagger"})]
     rel_path = generate_data(input_features, output_features, csv_filename)
 
     with pytest.raises(ValueError):
@@ -130,8 +126,8 @@ def test_experiment_seq_seq_tagger_fails_for_non_length_preserving_encoders(csv_
 
 def test_experiment_seq_seq_model_def_file(csv_filename, yaml_filename):
     # seq-to-seq test to use config file instead of dictionary
-    input_features = [text_feature(reduce_output=None, encoder="embed")]
-    output_features = [text_feature(reduce_input=None, vocab_size=3, decoder="tagger")]
+    input_features = [text_feature(encoder={"reduce_output": None, "type": "embed"})]
+    output_features = [text_feature(decoder={"reduce_input": None, "vocab_size": 3, "type": "tagger"})]
 
     # Save the config to a yaml file
     config = {
@@ -149,8 +145,8 @@ def test_experiment_seq_seq_model_def_file(csv_filename, yaml_filename):
 
 def test_experiment_seq_seq_train_test_valid(tmpdir):
     # seq-to-seq test to use train, test, validation files
-    input_features = [text_feature(reduce_output=None, encoder="rnn")]
-    output_features = [text_feature(reduce_input=None, vocab_size=3, decoder="tagger")]
+    input_features = [text_feature(encoder={"reduce_output": None, "type": "rnn"})]
+    output_features = [text_feature(decoder={"reduce_input": None, "vocab_size": 3, "type": "tagger"})]
 
     train_csv = generate_data(input_features, output_features, os.path.join(tmpdir, "train.csv"))
     test_csv = generate_data(input_features, output_features, os.path.join(tmpdir, "test.csv"), 20)
@@ -169,13 +165,16 @@ def test_experiment_seq_seq_train_test_valid(tmpdir):
 @pytest.mark.parametrize("encoder", ENCODERS)
 def test_experiment_multi_input_intent_classification(csv_filename, encoder):
     # Multiple inputs, Single category output
-    input_features = [text_feature(vocab_size=10, min_len=1, representation="sparse"), category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [
+        text_feature(encoder={"vocab_size": 10, "min_len": 1, "representation": "sparse"}),
+        category_feature(encoder={"vocab_size": 10}),
+    ]
+    output_features = [category_feature(decoder={"reduce_input": "sum", "vocab_size": 2})]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
 
-    input_features[0]["encoder"] = encoder
+    input_features[0][ENCODER][TYPE] = encoder
     run_experiment(input_features, output_features, dataset=rel_path)
 
 
@@ -189,8 +188,8 @@ def test_experiment_with_torch_module_dict_feature_name(csv_filename):
 
 def test_experiment_multiclass_with_class_weights(csv_filename):
     # Multiple inputs, Single category output
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=3, loss={"class_weights": [0, 1, 2, 3]})]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 3}, loss={"class_weights": [0, 1, 2]})]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -199,8 +198,8 @@ def test_experiment_multiclass_with_class_weights(csv_filename):
 
 def test_experiment_multilabel_with_class_weights(csv_filename):
     # Multiple inputs, Single category output
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [set_feature(vocab_size=3, loss={"class_weights": [0, 1, 2, 3]})]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [set_feature(decoder={"vocab_size": 3}, loss={"class_weights": [0, 1, 2, 3]})]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -212,20 +211,20 @@ def test_experiment_multilabel_with_class_weights(csv_filename):
     [
         # baseline test case
         [
-            category_feature(vocab_size=2, reduce_input="sum"),
-            sequence_feature(vocab_size=10, max_len=5),
+            category_feature(decoder={"reduce_input": "sum", "vocab_size": 2}),
+            sequence_feature(decoder={"vocab_size": 10, "max_len": 5}),
             number_feature(),
         ],
         # use generator as decoder
         [
-            category_feature(vocab_size=2, reduce_input="sum"),
-            sequence_feature(vocab_size=10, max_len=5, decoder="generator"),
+            category_feature(decoder={"vocab_size": 2, "reduce_input": "sum"}),
+            sequence_feature(decoder={"vocab_size": 10, "max_len": 5, "type": "generator"}),
             number_feature(),
         ],
         # Generator decoder and reduce_input = None
         [
-            category_feature(vocab_size=2, reduce_input="sum"),
-            sequence_feature(max_len=5, decoder="generator", reduce_input=None),
+            category_feature(decoder={"vocab_size": 2, "reduce_input": "sum"}),
+            sequence_feature(decoder={"max_len": 5, "reduce_input": None, "type": "generator"}),
             number_feature(normalization="minmax"),
         ],
         # output features with dependencies single dependency
@@ -241,11 +240,11 @@ def test_experiment_multilabel_with_class_weights(csv_filename):
 )
 def test_experiment_multiple_seq_seq(csv_filename, output_features):
     input_features = [
-        text_feature(vocab_size=100, min_len=1, encoder="stacked_cnn"),
+        text_feature(encoder={"vocab_size": 100, "min_len": 1, "type": "stacked_cnn"}),
         number_feature(normalization="zscore"),
-        category_feature(vocab_size=10, embedding_size=5),
+        category_feature(encoder={"vocab_size": 10, "embedding_size": 5}),
         set_feature(),
-        sequence_feature(vocab_size=10, max_len=10, encoder="embed"),
+        sequence_feature(encoder={"vocab_size": 10, "max_len": 10, "type": "embed"}),
     ]
     output_features = output_features
 
@@ -264,7 +263,6 @@ def test_basic_image_feature(num_channels, image_source, in_memory, skip_save_pr
     input_features = [
         image_feature(
             folder=image_dest_folder,
-            encoder="stacked_cnn",
             preprocessing={
                 "in_memory": in_memory,
                 "height": 12,
@@ -272,11 +270,14 @@ def test_basic_image_feature(num_channels, image_source, in_memory, skip_save_pr
                 "num_channels": num_channels,
                 "num_processes": 5,
             },
-            output_size=16,
-            num_filters=8,
+            encoder={
+                "type": "stacked_cnn",
+                "output_size": 16,
+                "num_filters": 8,
+            },
         )
     ]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    output_features = [category_feature(decoder={"reduce_input": "sum", "vocab_size": 2})]
 
     rel_path = generate_data(input_features, output_features, os.path.join(tmpdir, "dataset.csv"))
 
@@ -300,11 +301,11 @@ def test_experiment_infer_image_metadata(tmpdir):
 
     # Resnet encoder
     input_features = [
-        image_feature(folder=image_dest_folder, encoder="stacked_cnn", output_size=16, num_filters=8),
-        text_feature(encoder="embed", min_len=1),
+        image_feature(folder=image_dest_folder, encoder={"type": "stacked_cnn", "output_size": 16, "num_filters": 8}),
+        text_feature(encoder={"type": "embed", "min_len": 1}),
         number_feature(normalization="zscore"),
     ]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum"), number_feature()]
+    output_features = [category_feature(decoder={"reduce_input": "sum", "vocab_size": 2}), number_feature()]
 
     rel_path = generate_data(input_features, output_features, os.path.join(tmpdir, "dataset.csv"))
 
@@ -333,17 +334,15 @@ def test_experiment_image_inputs(image_params: ImageParams, tmpdir):
     input_features = [
         image_feature(
             folder=image_dest_folder,
-            encoder="resnet",
             preprocessing={"in_memory": True, "height": 12, "width": 12, "num_channels": 3, "num_processes": 5},
-            output_size=16,
-            num_filters=8,
+            encoder={"type": "resnet", "output_size": 16, "num_filters": 8},
         ),
-        text_feature(encoder="embed", min_len=1),
+        text_feature(encoder={"type": "embed", "min_len": 1}),
         number_feature(normalization="zscore"),
     ]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum"), number_feature()]
+    output_features = [category_feature(decoder={"reduce_input": "sum", "vocab_size": 2}), number_feature()]
 
-    input_features[0]["encoder"] = image_params.image_encoder
+    input_features[0]["encoder"]["type"] = image_params.image_encoder
     input_features[0]["preprocessing"]["in_memory"] = image_params.in_memory_flag
     rel_path = generate_data(input_features, output_features, os.path.join(tmpdir, "dataset.csv"))
 
@@ -368,14 +367,12 @@ def test_experiment_image_dataset(train_format, train_in_memory, test_format, te
     input_features = [
         image_feature(
             folder=image_dest_folder,
-            encoder="stacked_cnn",
             preprocessing={"in_memory": True, "height": 12, "width": 12, "num_channels": 3, "num_processes": 5},
-            output_size=16,
-            num_filters=8,
+            encoder={"type": "stacked_cnn", "output_size": 16, "num_filters": 8},
         ),
     ]
     output_features = [
-        category_feature(vocab_size=2, reduce_input="sum"),
+        category_feature(decoder={"reduce_input": "sum", "vocab_size": 2}),
     ]
 
     config = {
@@ -507,16 +504,18 @@ def test_experiment_audio_inputs(tmpdir):
 def test_experiment_tied_weights(csv_filename):
     # Single sequence input, single category output
     input_features = [
-        text_feature(name="text_feature1", min_len=1, encoder="cnnrnn", reduce_output="sum"),
-        text_feature(name="text_feature2", min_len=1, encoder="cnnrnn", reduce_output="sum", tied="text_feature1"),
+        text_feature(name="text_feature1", encoder={"min_len": 1, "type": "cnnrnn", "reduce_output": "sum"}),
+        text_feature(
+            name="text_feature2", encoder={"min_len": 1, "type": "cnnrnn", "reduce_output": "sum"}, tied="text_feature1"
+        ),
     ]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    output_features = [category_feature(decoder={"reduce_input": "sum", "vocab_size": 2})]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
     for encoder in ENCODERS:
-        input_features[0]["encoder"] = encoder
-        input_features[1]["encoder"] = encoder
+        input_features[0][ENCODER][TYPE] = encoder
+        input_features[1][ENCODER][TYPE] = encoder
         run_experiment(input_features, output_features, dataset=rel_path)
 
 
@@ -524,8 +523,12 @@ def test_experiment_tied_weights(csv_filename):
 @pytest.mark.parametrize("attention", [False, True])
 def test_sequence_tagger(enc_cell_type, attention, csv_filename):
     # Define input and output features
-    input_features = [sequence_feature(max_len=10, encoder="rnn", cell_type=enc_cell_type, reduce_output=None)]
-    output_features = [sequence_feature(max_len=10, decoder="tagger", attention=attention, reduce_input=None)]
+    input_features = [
+        sequence_feature(encoder={"max_len": 10, "type": "rnn", "cell_type": enc_cell_type, "reduce_output": None})
+    ]
+    output_features = [
+        sequence_feature(decoder={"max_len": 10, "type": "tagger", "reduce_input": None, "attention": attention})
+    ]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -536,8 +539,8 @@ def test_sequence_tagger(enc_cell_type, attention, csv_filename):
 
 def test_sequence_tagger_text(csv_filename):
     # Define input and output features
-    input_features = [text_feature(max_len=10, encoder="rnn", reduce_output=None)]
-    output_features = [sequence_feature(max_len=10, decoder="tagger", reduce_input=None)]
+    input_features = [text_feature(encoder={"max_len": 10, "type": "rnn", "reduce_output": None})]
+    output_features = [sequence_feature(decoder={"max_len": 10, "reduce_input": None, "type": "tagger"})]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -551,23 +554,27 @@ def test_experiment_sequence_combiner_with_reduction_fails(csv_filename):
         "input_features": [
             sequence_feature(
                 name="seq1",
-                min_len=5,
-                max_len=5,
-                encoder="embed",
-                cell_type="lstm",
-                reduce_output="sum",
+                encoder={
+                    "min_len": 5,
+                    "max_len": 5,
+                    "type": "embed",
+                    "cell_type": "lstm",
+                    "reduce_output": "sum",
+                },
             ),
             sequence_feature(
                 name="seq2",
-                min_len=5,
-                max_len=5,
-                encoder="embed",
-                cell_type="lstm",
-                reduce_output="sum",
+                encoder={
+                    "min_len": 5,
+                    "max_len": 5,
+                    "type": "embed",
+                    "cell_type": "lstm",
+                    "reduce_output": "sum",
+                },
             ),
-            category_feature(vocab_size=5),
+            category_feature(encoder={"vocab_size": 5}),
         ],
-        "output_features": [category_feature(reduce_input="sum", vocab_size=5)],
+        "output_features": [category_feature(decoder={"reduce_input": "sum", "vocab_size": 5})],
         TRAINER: {"epochs": 2},
         "combiner": {
             "type": "sequence",
@@ -591,14 +598,28 @@ def test_experiment_sequence_combiner(sequence_encoder, csv_filename):
     config = {
         "input_features": [
             sequence_feature(
-                name="seq1", min_len=5, max_len=5, encoder=sequence_encoder, cell_type="lstm", reduce_output=None
+                name="seq1",
+                encoder={
+                    "min_len": 5,
+                    "max_len": 5,
+                    "type": sequence_encoder,
+                    "cell_type": "lstm",
+                    "reduce_output": None,
+                },
             ),
             sequence_feature(
-                name="seq2", min_len=5, max_len=5, encoder=sequence_encoder, cell_type="lstm", reduce_output=None
+                name="seq2",
+                encoder={
+                    "min_len": 5,
+                    "max_len": 5,
+                    "type": sequence_encoder,
+                    "cell_type": "lstm",
+                    "reduce_output": None,
+                },
             ),
             category_feature(vocab_size=5),
         ],
-        "output_features": [category_feature(reduce_input="sum", vocab_size=5)],
+        "output_features": [category_feature(decoder={"reduce_input": "sum", "vocab_size": 5})],
         TRAINER: {"epochs": 2},
         "combiner": {
             "type": "sequence",
@@ -617,8 +638,8 @@ def test_experiment_sequence_combiner(sequence_encoder, csv_filename):
 def test_experiment_model_resume(tmpdir):
     # Single sequence input, single category output
     # Tests saving a model file, loading it to rerun training and predict
-    input_features = [sequence_feature(encoder="rnn", reduce_output="sum")]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [sequence_feature(encoder={"type": "rnn", "reduce_output": "sum"})]
+    output_features = [category_feature(decoder={"reduce_input": "sum", "vocab_size": 2})]
     # Generate test data
     rel_path = generate_data(input_features, output_features, os.path.join(tmpdir, "dataset.csv"))
 
@@ -639,7 +660,7 @@ def test_experiment_model_resume(tmpdir):
 
 def test_experiment_various_feature_types(csv_filename):
     input_features = [binary_feature(), bag_feature()]
-    output_features = [set_feature(max_len=3, vocab_size=5)]
+    output_features = [set_feature(decoder={"max_len": 3, "vocab_size": 5})]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -652,7 +673,7 @@ def test_experiment_timeseries(csv_filename):
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
-    input_features[0]["encoder"] = "transformer"
+    input_features[0][ENCODER][TYPE] = "transformer"
     run_experiment(input_features, output_features, dataset=rel_path)
 
 
@@ -661,14 +682,12 @@ def test_visual_question_answering(tmpdir):
     input_features = [
         image_feature(
             folder=image_dest_folder,
-            encoder="resnet",
             preprocessing={"in_memory": True, "height": 8, "width": 8, "num_channels": 3, "num_processes": 5},
-            output_size=8,
-            num_filters=8,
+            encoder={"type": "resnet", "output_size": 8, "num_filters": 8},
         ),
-        text_feature(encoder="embed", min_len=1),
+        text_feature(encoder={"type": "embed", "min_len": 1}),
     ]
-    output_features = [sequence_feature(decoder="generator", cell_type="lstm")]
+    output_features = [sequence_feature(decoder={"type": "generator", "cell_type": "lstm"})]
     rel_path = generate_data(input_features, output_features, os.path.join(tmpdir, "dataset.csv"))
 
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -688,12 +707,10 @@ def test_image_resizing_num_channel_handling(tmpdir):
     input_features = [
         image_feature(
             folder=image_dest_folder,
-            encoder="resnet",
             preprocessing={"in_memory": True, "height": 8, "width": 8, "num_channels": 3, "num_processes": 5},
-            output_size=8,
-            num_filters=8,
+            encoder={"type": "resnet", "output_size": 8, "num_filters": 8},
         ),
-        text_feature(encoder="embed", min_len=1),
+        text_feature(encoder={"type": "embed", "min_len": 1}),
         number_feature(normalization="minmax"),
     ]
     output_features = [binary_feature(), number_feature()]
@@ -720,12 +737,12 @@ def test_image_resizing_num_channel_handling(tmpdir):
 @pytest.mark.parametrize("encoder", ["wave", "embed"])
 def test_experiment_date(encoder, csv_filename):
     input_features = [date_feature()]
-    output_features = [category_feature(vocab_size=2)]
+    output_features = [category_feature(decoder={"vocab_size": 2})]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
 
-    input_features[0]["encoder"] = encoder
+    input_features[0][ENCODER] = {TYPE: encoder}
     run_experiment(input_features, output_features, dataset=rel_path)
 
 
@@ -737,7 +754,7 @@ def test_experiment_h3(encoder, csv_filename):
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
 
-    input_features[0]["encoder"] = encoder
+    input_features[0][ENCODER] = {TYPE: encoder}
     run_experiment(input_features, output_features, dataset=rel_path)
 
 
