@@ -24,11 +24,12 @@ import os
 import random
 import subprocess
 
+import numpy as np
 import pytest
 
-from ludwig.constants import TRAINER
+from ludwig.constants import ENCODER, TRAINER, TYPE
 from ludwig.experiment import experiment_cli
-from ludwig.globals import PREDICTIONS_PARQUET_FILE_NAME, TEST_STATISTICS_FILE_NAME
+from ludwig.globals import DESCRIPTION_FILE_NAME, PREDICTIONS_PARQUET_FILE_NAME, TEST_STATISTICS_FILE_NAME
 from ludwig.utils.data_utils import get_split_path
 from ludwig.visualize import _extract_ground_truth_values
 from tests.integration_tests.test_visualization_api import obtain_df_splits
@@ -80,7 +81,7 @@ def get_output_feature_name(experiment_dir, output_feature=0):
     :return output_feature_name: name of the first output feature name
                         from the experiment
     """
-    description_file = os.path.join(experiment_dir, "description.json")
+    description_file = os.path.join(experiment_dir, DESCRIPTION_FILE_NAME)
     with open(description_file, "rb") as f:
         content = json.load(f)
     output_feature_name = content["config"]["output_features"][output_feature]["name"]
@@ -93,12 +94,12 @@ def test_visualization_learning_curves_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [text_feature(encoder="parallel_cnn")]
+    input_features = [text_feature(encoder={"type": "parallel_cnn"})]
     output_features = [category_feature()]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
-    input_features[0]["encoder"] = "parallel_cnn"
+    input_features[0][ENCODER][TYPE] = "parallel_cnn"
     exp_dir_name = run_experiment_with_visualization(input_features, output_features, dataset=rel_path)
 
     vis_output_pattern_pdf = os.path.join(exp_dir_name, "*.pdf")
@@ -136,12 +137,12 @@ def test_visualization_confusion_matrix_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [text_feature(encoder="parallel_cnn")]
+    input_features = [text_feature(encoder={"type": "parallel_cnn"})]
     output_features = [category_feature()]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
-    input_features[0]["encoder"] = "parallel_cnn"
+    input_features[0][ENCODER][TYPE] = "parallel_cnn"
     exp_dir_name = run_experiment_with_visualization(input_features, output_features, dataset=rel_path)
     vis_output_pattern_pdf = os.path.join(exp_dir_name, "*.pdf")
     vis_output_pattern_png = os.path.join(exp_dir_name, "*.png")
@@ -182,12 +183,12 @@ def test_visualization_compare_performance_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [text_feature(encoder="parallel_cnn")]
+    input_features = [text_feature(encoder={"type": "parallel_cnn"})]
     output_features = [category_feature()]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
-    input_features[0]["encoder"] = "parallel_cnn"
+    input_features[0][ENCODER][TYPE] = "parallel_cnn"
     exp_dir_name = run_experiment_with_visualization(input_features, output_features, dataset=rel_path)
     vis_output_pattern_pdf = os.path.join(exp_dir_name, "*.pdf")
     vis_output_pattern_png = os.path.join(exp_dir_name, "*.png")
@@ -228,8 +229,8 @@ def test_visualization_compare_classifiers_from_prob_csv_output_saved(csv_filena
     :param csv_filename: csv fixture from tests.fixtures.filenames.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -285,8 +286,8 @@ def test_visualization_compare_classifiers_from_prob_npy_output_saved(csv_filena
     :param csv_filename: csv fixture from tests.fixtures.filenames.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -298,7 +299,7 @@ def test_visualization_compare_classifiers_from_prob_npy_output_saved(csv_filena
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -342,8 +343,8 @@ def test_visualization_compare_classifiers_from_pred_npy_output_saved(csv_filena
     :param csv_filename: csv fixture from tests.fixtures.filenames.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -354,7 +355,7 @@ def test_visualization_compare_classifiers_from_pred_npy_output_saved(csv_filena
     prediction = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     ground_truth_metadata = experiment_source_data_name + ".meta.json"
     test_cmd_pdf = [
         "python",
@@ -399,8 +400,8 @@ def test_visualization_compare_classifiers_from_pred_csv_output_saved(csv_filena
     :param csv_filename: csv fixture from tests.fixtures.filenames.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -411,7 +412,7 @@ def test_visualization_compare_classifiers_from_pred_csv_output_saved(csv_filena
     prediction = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     ground_truth_metadata = experiment_source_data_name + ".meta.json"
     test_cmd_pdf = [
         "python",
@@ -455,8 +456,8 @@ def test_visualization_compare_classifiers_subset_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -467,7 +468,7 @@ def test_visualization_compare_classifiers_subset_output_saved(csv_filename):
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -508,8 +509,8 @@ def test_visualization_compare_classifiers_subset_output_saved(csv_filename):
 
 def test_visualization_compare_classifiers_changing_k_output_pdf(csv_filename):
     """It should be possible to save figures as pdf in the specified directory."""
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -520,7 +521,7 @@ def test_visualization_compare_classifiers_changing_k_output_pdf(csv_filename):
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     ground_truth_metadata = exp_dir_name + "/model/training_set_metadata.json"
     test_cmd_pdf = [
         "python",
@@ -565,8 +566,8 @@ def test_visualization_compare_classifiers_multiclass_multimetric_output_saved(c
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -613,8 +614,8 @@ def test_visualization_compare_classifiers_predictions_npy_output_saved(csv_file
     :param csv_filename: csv fixture from tests.fixtures.filenames.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -625,7 +626,7 @@ def test_visualization_compare_classifiers_predictions_npy_output_saved(csv_file
     prediction = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -669,8 +670,8 @@ def test_visualization_compare_classifiers_predictions_csv_output_saved(csv_file
     :param csv_filename: csv fixture from tests.fixtures.filenames.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -681,7 +682,7 @@ def test_visualization_compare_classifiers_predictions_csv_output_saved(csv_file
     prediction = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -724,8 +725,8 @@ def test_visualization_cmp_classifiers_predictions_distribution_output_saved(csv
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -736,7 +737,7 @@ def test_visualization_cmp_classifiers_predictions_distribution_output_saved(csv
     prediction = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -779,8 +780,8 @@ def test_visualization_cconfidence_thresholding_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -791,7 +792,7 @@ def test_visualization_cconfidence_thresholding_output_saved(csv_filename):
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -834,8 +835,8 @@ def test_visualization_confidence_thresholding_data_vs_acc_output_saved(csv_file
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -846,7 +847,7 @@ def test_visualization_confidence_thresholding_data_vs_acc_output_saved(csv_file
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -889,8 +890,8 @@ def test_visualization_confidence_thresholding_data_vs_acc_subset_output_saved(c
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -901,7 +902,7 @@ def test_visualization_confidence_thresholding_data_vs_acc_subset_output_saved(c
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -946,8 +947,8 @@ def test_vis_confidence_thresholding_data_vs_acc_subset_per_class_output_saved(c
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=5, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 5}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -958,7 +959,7 @@ def test_vis_confidence_thresholding_data_vs_acc_subset_per_class_output_saved(c
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -1006,19 +1007,19 @@ def test_vis_confidence_thresholding_2thresholds_2d_output_saved(csv_filename):
     :return: None
     """
     input_features = [
-        text_feature(vocab_size=10, min_len=1, encoder="stacked_cnn"),
+        text_feature(encoder={"vocab_size": 10, "min_len": 1, "type": "stacked_cnn"}),
         number_feature(),
-        category_feature(vocab_size=10, embedding_size=5),
+        category_feature(encoder={"vocab_size": 10, "embedding_size": 5}),
         set_feature(),
-        sequence_feature(vocab_size=10, max_len=10, encoder="embed"),
+        sequence_feature(encoder={"vocab_size": 10, "max_len": 10, "type": "embed"}),
     ]
     output_features = [
-        category_feature(vocab_size=2, reduce_input="sum"),
-        category_feature(vocab_size=2, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
     ]
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
-    input_features[0]["encoder"] = "parallel_cnn"
+    input_features[0][ENCODER][TYPE] = "parallel_cnn"
     exp_dir_name = run_experiment_with_visualization(input_features, output_features, dataset=rel_path)
     vis_output_pattern_pdf = os.path.join(exp_dir_name, "*.pdf")
     vis_output_pattern_png = os.path.join(exp_dir_name, "*.png")
@@ -1027,7 +1028,7 @@ def test_vis_confidence_thresholding_2thresholds_2d_output_saved(csv_filename):
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -1072,19 +1073,19 @@ def test_vis_confidence_thresholding_2thresholds_3d_output_saved(csv_filename):
     :return: None
     """
     input_features = [
-        text_feature(vocab_size=10, min_len=1, encoder="stacked_cnn"),
+        text_feature(encoder={"vocab_size": 10, "min_len": 1, "type": "stacked_cnn"}),
         number_feature(),
-        category_feature(vocab_size=10, embedding_size=5),
+        category_feature(encoder={"vocab_size": 10, "embedding_size": 5}),
         set_feature(),
-        sequence_feature(vocab_size=10, max_len=10, encoder="embed"),
+        sequence_feature(encoder={"vocab_size": 10, "max_len": 10, "type": "embed"}),
     ]
     output_features = [
-        category_feature(vocab_size=2, reduce_input="sum"),
-        category_feature(vocab_size=2, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
     ]
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
-    input_features[0]["encoder"] = "parallel_cnn"
+    input_features[0][ENCODER][TYPE] = "parallel_cnn"
     exp_dir_name = run_experiment_with_visualization(input_features, output_features, dataset=rel_path)
     vis_output_pattern_pdf = os.path.join(exp_dir_name, "*.pdf")
     vis_output_pattern_png = os.path.join(exp_dir_name, "*.png")
@@ -1093,7 +1094,7 @@ def test_vis_confidence_thresholding_2thresholds_3d_output_saved(csv_filename):
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -1137,21 +1138,21 @@ def test_visualization_binary_threshold_vs_metric_output_saved(csv_filename, bin
     :return: None
     """
     input_features = [
-        text_feature(vocab_size=10, min_len=1, encoder="stacked_cnn"),
+        text_feature(encoder={"vocab_size": 10, "min_len": 1, "type": "stacked_cnn"}),
         number_feature(),
-        category_feature(vocab_size=10, embedding_size=5),
+        category_feature(encoder={"vocab_size": 10, "embedding_size": 5}),
         set_feature(),
-        sequence_feature(vocab_size=10, max_len=10, encoder="embed"),
+        sequence_feature(encoder={"vocab_size": 10, "max_len": 10, "type": "embed"}),
     ]
     if binary_output_type:
         output_features = [binary_feature()]
     else:
-        output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+        output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     random.seed(1919)
     rel_path = generate_data(input_features, output_features, csv_filename)
-    input_features[0]["encoder"] = "parallel_cnn"
+    input_features[0][ENCODER][TYPE] = "parallel_cnn"
     exp_dir_name = run_experiment_with_visualization(input_features, output_features, dataset=rel_path)
     vis_output_pattern_pdf = os.path.join(exp_dir_name, "*.pdf")
     vis_output_pattern_png = os.path.join(exp_dir_name, "*.png")
@@ -1159,7 +1160,7 @@ def test_visualization_binary_threshold_vs_metric_output_saved(csv_filename, bin
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -1167,7 +1168,7 @@ def test_visualization_binary_threshold_vs_metric_output_saved(csv_filename, bin
         "--visualization",
         "binary_threshold_vs_metric",
         "--positive_label",
-        "2",
+        "1",
         "--metrics",
         "accuracy",
         "precision",
@@ -1210,11 +1211,11 @@ def test_visualization_roc_curves_output_saved(csv_filename, binary_output_type)
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
     if binary_output_type:
         output_features = [binary_feature()]
     else:
-        output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+        output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -1225,7 +1226,7 @@ def test_visualization_roc_curves_output_saved(csv_filename, binary_output_type)
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -1233,7 +1234,7 @@ def test_visualization_roc_curves_output_saved(csv_filename, binary_output_type)
         "--visualization",
         "roc_curves",
         "--positive_label",
-        "2",
+        "1",
         "--metrics",
         "accuracy",
         "--ground_truth",
@@ -1316,8 +1317,8 @@ def test_visualization_calibration_1_vs_all_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -1328,7 +1329,7 @@ def test_visualization_calibration_1_vs_all_output_saved(csv_filename):
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -1366,7 +1367,7 @@ def test_visualization_calibration_1_vs_all_output_saved(csv_filename):
         figure_cnt = glob.glob(viz_pattern)
 
         assert 0 == result.returncode
-        assert 7 == len(figure_cnt)
+        assert 5 == len(figure_cnt)
 
 
 def test_visualization_calibration_multiclass_output_saved(csv_filename):
@@ -1375,8 +1376,8 @@ def test_visualization_calibration_multiclass_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -1387,7 +1388,7 @@ def test_visualization_calibration_multiclass_output_saved(csv_filename):
     probability = os.path.join(exp_dir_name, PREDICTIONS_PARQUET_FILE_NAME)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
     test_cmd_pdf = [
         "python",
         "-m",
@@ -1430,8 +1431,8 @@ def test_visualization_frequency_vs_f1_output_saved(csv_filename):
     :param csv_filename: csv fixture from tests.conftest.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -1474,14 +1475,44 @@ def test_visualization_frequency_vs_f1_output_saved(csv_filename):
         assert 2 == len(figure_cnt)
 
 
+def test_load_ground_truth_split_from_df(csv_filename):
+    import pandas as pd
+
+    ground_truth = pd.DataFrame(
+        {
+            "PassengerId": [1],
+            "Survived": [0],
+            "Pclass": [3],
+            "Name": ["Braund, Mr. Owen Harris"],
+            "Sex": ["male"],
+            "Age": [22.0],
+            "SibSp": [1],
+            "Parch": [0],
+            "Ticket": ["A/5 21171"],
+            "Fare": ["7.25"],
+            "Cabin": [None],
+            "Embarked": ["S"],
+            "split": [0],
+        }
+    )
+    output_feature = "Survived"
+    ground_truth_train_split = _extract_ground_truth_values(ground_truth, output_feature, 0)
+    ground_truth_val_split = _extract_ground_truth_values(ground_truth, output_feature, 1)
+    ground_truth_test_split = _extract_ground_truth_values(ground_truth, output_feature, 2)
+
+    assert ground_truth_train_split.equals(pd.Series([0]))
+    assert ground_truth_val_split.empty
+    assert ground_truth_test_split.empty
+
+
 def test_load_ground_truth_split_from_file(csv_filename):
     """Ensure correct ground truth split is loaded when ground_truth_split is given.
 
     :param csv_filename: csv fixture from tests.fixtures.filenames.csv_filename
     :return: None
     """
-    input_features = [category_feature(vocab_size=10)]
-    output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+    input_features = [category_feature(encoder={"vocab_size": 10})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -1489,7 +1520,7 @@ def test_load_ground_truth_split_from_file(csv_filename):
     output_feature_name = get_output_feature_name(exp_dir_name)
     experiment_source_data_name = csv_filename.split(".")[0]
     ground_truth = experiment_source_data_name + ".csv"
-    split_file = experiment_source_data_name + ".split.csv"
+    split_file = experiment_source_data_name + ".split.parquet"
 
     # retrieve ground truth from source data set
     ground_truth_train_split = _extract_ground_truth_values(ground_truth, output_feature_name, 0, split_file)
@@ -1501,6 +1532,6 @@ def test_load_ground_truth_split_from_file(csv_filename):
     target_predictions_from_val = val_df[output_feature_name]
     target_predictions_from_test = test_df[output_feature_name]
 
-    assert str(ground_truth_train_split.values) == str(target_predictions_from_train.values)
-    assert str(ground_truth_val_split.values) == str(target_predictions_from_val.values)
-    assert str(ground_truth_test_split.values) == str(target_predictions_from_test.values)
+    assert np.all(ground_truth_train_split.eq(target_predictions_from_train))
+    assert np.all(ground_truth_val_split.eq(target_predictions_from_val))
+    assert np.all(ground_truth_test_split.eq(target_predictions_from_test))
