@@ -18,19 +18,31 @@ import os
 from tempfile import TemporaryDirectory
 
 import numpy as np
-import pandas as pd
 import pytest
 
 from ludwig import visualize
 from ludwig.api import LudwigModel
-from ludwig.constants import NAME, PREDICTIONS, PROBABILITIES, PROBABILITY, SPLIT, TEST, TRAINER, TRAINING, VALIDATION
-from ludwig.data.preprocessing import get_split
-from ludwig.utils.data_utils import read_csv, split_dataset_ttv
+from ludwig.constants import (
+    ENCODER,
+    NAME,
+    PREDICTIONS,
+    PROBABILITIES,
+    PROBABILITY,
+    TEST,
+    TRAINER,
+    TRAINING,
+    TYPE,
+    VALIDATION,
+)
+from ludwig.data.split import get_splitter
+from ludwig.globals import HYPEROPT_STATISTICS_FILE_NAME
+from ludwig.utils.data_utils import read_csv
 from tests.integration_tests.utils import (
     bag_feature,
     binary_feature,
     category_feature,
     generate_data,
+    LocalTestBackend,
     number_feature,
     sequence_feature,
     set_feature,
@@ -72,8 +84,8 @@ class Experiment:
     def __init__(self, csv_filename, tmpdir):
         self.tmpdir = tmpdir
         self.csv_file = os.path.join(tmpdir, csv_filename)
-        self.input_features = [category_feature(vocab_size=10)]
-        self.output_features = [category_feature(vocab_size=2, reduce_input="sum")]
+        self.input_features = [category_feature(encoder={"vocab_size": 10})]
+        self.output_features = [category_feature(decoder={"vocab_size": 2}, reduce_input="sum")]
         data_csv = generate_data(self.input_features, self.output_features, self.csv_file)
         self.model = self._create_model()
         test_df, train_df, val_df = obtain_df_splits(data_csv)
@@ -122,12 +134,8 @@ def obtain_df_splits(data_csv):
     data_df = read_csv(data_csv)
     # Obtain data split array mapping data rows to split type
     # 0-train, 1-validation, 2-test
-    data_df[SPLIT] = get_split(data_df)
-    train_split, test_split, val_split = split_dataset_ttv(data_df, SPLIT)
-    # Splits are python dictionaries not dataframes- they need to be converted.
-    test_df = pd.DataFrame(test_split)
-    train_df = pd.DataFrame(train_split)
-    val_df = pd.DataFrame(val_split)
+    splitter = get_splitter("random")
+    train_df, val_df, test_df = splitter.split(data_df, LocalTestBackend())
     return test_df, train_df, val_df
 
 
@@ -475,7 +483,7 @@ def test_confidence_thresholding_data_vs_acc_subset_per_class_vis_api(experiment
                 experiment.ground_truth,
                 experiment.ground_truth_metadata,
                 experiment.output_feature_name,
-                top_n_classes=[3],
+                top_n_classes=[2],
                 labels_limit=0,
                 subset="ground_truth",
                 model_names=["Model1", "Model2"],
@@ -485,7 +493,7 @@ def test_confidence_thresholding_data_vs_acc_subset_per_class_vis_api(experiment
             figure_cnt = glob.glob(vis_output_pattern_pdf)
             # 3 figures should be saved because experiment setting top_n_classes = 3
             # hence one figure per class
-            assert 3 == len(figure_cnt)
+            assert 2 == len(figure_cnt)
 
 
 def test_confidence_thresholding_2thresholds_2d_vis_api(csv_filename):
@@ -495,21 +503,21 @@ def test_confidence_thresholding_2thresholds_2d_vis_api(csv_filename):
     :return: None
     """
     input_features = [
-        text_feature(vocab_size=10, min_len=1, encoder="stacked_cnn"),
+        text_feature(encoder={"vocab_size": 10, "min_len": 1, "type": "stacked_cnn"}),
         number_feature(),
-        category_feature(vocab_size=10, embedding_size=5),
+        category_feature(encoder={"vocab_size": 10, "embedding_size": 5}),
         set_feature(),
-        sequence_feature(vocab_size=10, max_len=10, encoder="embed"),
+        sequence_feature(encoder={"vocab_size": 10, "max_len": 10, "type": "embed"}),
     ]
     output_features = [
-        category_feature(vocab_size=2, reduce_input="sum"),
-        category_feature(vocab_size=2, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
     ]
     encoder = "parallel_cnn"
     with TemporaryDirectory() as tmpvizdir:
         # Generate test data
         data_csv = generate_data(input_features, output_features, os.path.join(tmpvizdir, csv_filename))
-        input_features[0]["encoder"] = encoder
+        input_features[0][ENCODER][TYPE] = encoder
         model = run_api_experiment(input_features, output_features)
         test_df, train_df, val_df = obtain_df_splits(data_csv)
         _, _, output_dir = model.train(
@@ -567,21 +575,21 @@ def test_confidence_thresholding_2thresholds_3d_vis_api(csv_filename):
     :return: None
     """
     input_features = [
-        text_feature(vocab_size=10, min_len=1, encoder="stacked_cnn"),
+        text_feature(encoder={"vocab_size": 10, "min_len": 1, "type": "stacked_cnn"}),
         number_feature(),
-        category_feature(vocab_size=10, embedding_size=5),
+        category_feature(encoder={"vocab_size": 10, "embedding_size": 5}),
         set_feature(),
-        sequence_feature(vocab_size=10, max_len=10, encoder="embed"),
+        sequence_feature(encoder={"vocab_size": 10, "max_len": 10, "type": "embed"}),
     ]
     output_features = [
-        category_feature(vocab_size=2, reduce_input="sum"),
-        category_feature(vocab_size=2, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
+        category_feature(decoder={"vocab_size": 2}, reduce_input="sum"),
     ]
     encoder = "parallel_cnn"
     with TemporaryDirectory() as tmpvizdir:
         # Generate test data
         data_csv = generate_data(input_features, output_features, os.path.join(tmpvizdir, csv_filename))
-        input_features[0]["encoder"] = encoder
+        input_features[0][ENCODER][TYPE] = encoder
         model = run_api_experiment(input_features, output_features)
         test_df, train_df, val_df = obtain_df_splits(data_csv)
         _, _, output_dir = model.train(
@@ -644,7 +652,7 @@ def test_binary_threshold_vs_metric_vis_api(experiment_to_use):
     probabilities = experiment.probabilities
     viz_outputs = ("pdf", "png")
     metrics = ["accuracy"]
-    positive_label = 2
+    positive_label = 1
     with TemporaryDirectory() as tmpvizdir:
         for viz_output in viz_outputs:
             vis_output_pattern_pdf = tmpvizdir + f"/*.{viz_output}"
@@ -673,7 +681,7 @@ def test_roc_curves_vis_api(experiment_to_use):
     experiment = experiment_to_use
     probabilities = experiment.probabilities
     viz_outputs = ("pdf", "png")
-    positive_label = 2
+    positive_label = 1
     with TemporaryDirectory() as tmpvizdir:
         for viz_output in viz_outputs:
             vis_output_pattern_pdf = tmpvizdir + f"/*.{viz_output}"
@@ -750,7 +758,7 @@ def test_calibration_1_vs_all_vis_api(experiment_to_use):
                 file_format=viz_output,
             )
             figure_cnt = glob.glob(vis_output_pattern_pdf)
-            assert 7 == len(figure_cnt)
+            assert 5 == len(figure_cnt)
 
 
 def test_calibration_multiclass_vis_api(experiment_to_use):
@@ -839,7 +847,7 @@ def test_frequency_vs_f1_vis_api(experiment_to_use):
 def test_hyperopt_report_vis_api(hyperopt_results, tmpdir):
     vis_dir = os.path.join(tmpdir, "visualizations")
 
-    visualize.hyperopt_report(os.path.join(hyperopt_results, "hyperopt_statistics.json"), output_directory=vis_dir)
+    visualize.hyperopt_report(os.path.join(hyperopt_results, HYPEROPT_STATISTICS_FILE_NAME), output_directory=vis_dir)
 
     # test for creation of output directory
     assert os.path.isdir(vis_dir)
@@ -852,7 +860,7 @@ def test_hyperopt_report_vis_api(hyperopt_results, tmpdir):
 def test_hyperopt_hiplot_vis_api(hyperopt_results, tmpdir):
     vis_dir = os.path.join(tmpdir, "visualizations")
 
-    visualize.hyperopt_hiplot(os.path.join(hyperopt_results, "hyperopt_statistics.json"), output_directory=vis_dir)
+    visualize.hyperopt_hiplot(os.path.join(hyperopt_results, HYPEROPT_STATISTICS_FILE_NAME), output_directory=vis_dir)
 
     # test for creation of output directory
     assert os.path.isdir(vis_dir)
