@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-import contextlib
+
 import logging
 import multiprocessing
 import os
@@ -30,22 +30,17 @@ import cloudpickle
 import numpy as np
 import pandas as pd
 import torch
+from PIL import Image
 
 from ludwig.api import LudwigModel
 from ludwig.backend import LocalBackend
-from ludwig.constants import COLUMN, NAME, PROC_COLUMN, TRAINER, VECTOR
+from ludwig.constants import COLUMN, DECODER, ENCODER, NAME, PROC_COLUMN, TRAINER, VECTOR
 from ludwig.data.dataset_synthesizer import build_synthetic_dataset, DATETIME_FORMATS
 from ludwig.experiment import experiment_cli
 from ludwig.features.feature_utils import compute_feature_hash
-from ludwig.models.trainer import Trainer
+from ludwig.trainers.trainer import Trainer
 from ludwig.utils.data_utils import read_csv, replace_file_extension
 
-try:
-    import ray
-except ImportError:
-    ray = None
-
-
 logger = logging.getLogger(__name__)
 
 # Used in sequence-related unit tests (encoders, features) as well as end-to-end integration tests.
@@ -74,7 +69,6 @@ HF_ENCODERS = [
     "mt5",
 ]
 
-
 RAY_BACKEND_CONFIG = {
     "type": "ray",
     "processor": {
@@ -91,48 +85,6 @@ RAY_BACKEND_CONFIG = {
 }
 
 
-@contextlib.contextmanager
-def ray_start(num_cpus=2, num_gpus=None):
-    res = ray.init(
-        num_cpus=num_cpus,
-        num_gpus=num_gpus,
-        include_dashboard=False,
-        object_store_memory=150 * 1024 * 1024,
-    )
-    try:
-        yield res
-    finally:
-        ray.shutdown()
-
-
-@contextlib.contextmanager
-def ray_cluster():
-    ray.init(
-        num_cpus=2,
-        include_dashboard=False,
-        object_store_memory=150 * 1024 * 1024,
-    )
-    try:
-        yield
-    finally:
-        ray.shutdown()
-
-
-@contextlib.contextmanager
-def init_backend(backend: str, num_cpus=2, num_gpus=None):
-    if backend == "local":
-        with contextlib.nullcontext():
-            yield
-            return
-
-    if backend == "ray":
-        with ray_start(num_cpus=num_cpus, num_gpus=num_gpus):
-            yield
-            return
-
-    raise ValueError(f"Unrecognized backend: {backend}")
-
-
 class LocalTestBackend(LocalBackend):
     @property
     def supports_multiprocessing(self):
@@ -141,7 +93,7 @@ class LocalTestBackend(LocalBackend):
 
 # Simulates running training on a separate node from the driver process
 class FakeRemoteBackend(LocalBackend):
-    def create_trainer(self, **kwargs):
+    def create_trainer(self, **kwargs) -> "BaseTrainer":  # noqa: F821
         return FakeRemoteTrainer(**kwargs)
 
     @property
@@ -197,6 +149,7 @@ def generate_data(
     :param input_features: schema
     :param output_features: schema
     :param filename: path to the file where data is stored
+    :param nan_percent: percent of values in a feature to be NaN
     :return:
     """
     features = input_features + output_features
@@ -211,64 +164,146 @@ def generate_data(
     return filename
 
 
+def recursive_update(dictionary, values):
+    for k, v in values.items():
+        if isinstance(v, dict):
+            dictionary[k] = recursive_update(dictionary.get(k, {}), v)
+        else:
+            dictionary[k] = v
+    return dictionary
+
+
 def random_string(length=5):
     return uuid.uuid4().hex[:length].upper()
 
 
 def number_feature(normalization=None, **kwargs):
-    feature = {"name": "num_" + random_string(), "type": "number", "preprocessing": {"normalization": normalization}}
-    feature.update(kwargs)
+    feature = {
+        "name": "num_" + random_string(),
+        "type": "number",
+        "preprocessing": {"normalization": normalization},
+    }
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
-def category_feature(**kwargs):
-    feature = {"type": "category", "name": "category_" + random_string(), "vocab_size": 10, "embedding_size": 5}
-    feature.update(kwargs)
+def category_feature(output_feature=False, **kwargs):
+    if DECODER in kwargs:
+        output_feature = True
+    feature = {
+        "type": "category",
+        "name": "category_" + random_string(),
+    }
+    if output_feature:
+        feature.update(
+            {
+                DECODER: {"type": "classifier", "vocab_size": 10},
+            }
+        )
+    else:
+        feature.update(
+            {
+                ENCODER: {"type": "dense", "vocab_size": 10, "embedding_size": 5},
+            }
+        )
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
-def text_feature(**kwargs):
+def text_feature(output_feature=False, **kwargs):
+    if DECODER in kwargs:
+        output_feature = True
     feature = {
         "name": "text_" + random_string(),
         "type": "text",
-        "vocab_size": 5,
-        "min_len": 7,
-        "max_len": 7,
-        "embedding_size": 8,
-        "state_size": 8,
     }
-    feature.update(kwargs)
+    if output_feature:
+        feature.update(
+            {
+                DECODER: {"type": "generator", "vocab_size": 5, "max_len": 7},
+            }
+        )
+    else:
+        feature.update(
+            {
+                ENCODER: {
+                    "type": "parallel_cnn",
+                    "vocab_size": 5,
+                    "min_len": 7,
+                    "max_len": 7,
+                    "embedding_size": 8,
+                    "state_size": 8,
+                },
+            }
+        )
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
-def set_feature(**kwargs):
-    feature = {"type": "set", "name": "set_" + random_string(), "vocab_size": 10, "max_len": 5, "embedding_size": 5}
-    feature.update(kwargs)
+def set_feature(output_feature=False, **kwargs):
+    if DECODER in kwargs:
+        output_feature = True
+    feature = {
+        "type": "set",
+        "name": "set_" + random_string(),
+    }
+    if output_feature:
+        feature.update(
+            {
+                DECODER: {"type": "classifier", "vocab_size": 10, "max_len": 5},
+            }
+        )
+    else:
+        feature.update(
+            {
+                ENCODER: {"type": "embed", "vocab_size": 10, "max_len": 5, "embedding_size": 5},
+            }
+        )
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
-def sequence_feature(**kwargs):
+def sequence_feature(output_feature=False, **kwargs):
+    if DECODER in kwargs:
+        output_feature = True
     feature = {
         "type": "sequence",
         "name": "sequence_" + random_string(),
-        "vocab_size": 10,
-        "max_len": 7,
-        "encoder": "embed",
-        "embedding_size": 8,
-        "output_size": 8,
-        "state_size": 8,
-        "num_filters": 8,
-        "hidden_size": 8,
     }
-    feature.update(kwargs)
+    if output_feature:
+        feature.update(
+            {
+                DECODER: {
+                    "type": "generator",
+                    "vocab_size": 10,
+                    "max_len": 7,
+                }
+            }
+        )
+    else:
+        feature.update(
+            {
+                ENCODER: {
+                    "type": "embed",
+                    "vocab_size": 10,
+                    "max_len": 7,
+                    "embedding_size": 8,
+                    "output_size": 8,
+                    "state_size": 8,
+                    "num_filters": 8,
+                    "hidden_size": 8,
+                },
+            }
+        )
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
@@ -278,14 +313,16 @@ def image_feature(folder, **kwargs):
     feature = {
         "type": "image",
         "name": "image_" + random_string(),
-        "encoder": "resnet",
         "preprocessing": {"in_memory": True, "height": 12, "width": 12, "num_channels": 3},
-        "resnet_size": 8,
+        ENCODER: {
+            "type": "resnet",
+            "resnet_size": 8,
+            "num_filters": 8,
+            "output_size": 8,
+        },
         "destination_folder": folder,
-        "output_size": 8,
-        "num_filters": 8,
     }
-    feature.update(kwargs)
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
@@ -296,48 +333,59 @@ def audio_feature(folder, **kwargs):
         "name": "audio_" + random_string(),
         "type": "audio",
         "preprocessing": {
-            "audio_feature": {
-                "type": "fbank",
-                "window_length_in_s": 0.04,
-                "window_shift_in_s": 0.02,
-                "num_filter_bands": 80,
-            },
+            "type": "fbank",
+            "window_length_in_s": 0.04,
+            "window_shift_in_s": 0.02,
+            "num_filter_bands": 80,
             "audio_file_length_limit_in_s": 3.0,
         },
-        "encoder": "stacked_cnn",
-        "should_embed": False,
-        "conv_layers": [
-            {"filter_size": 400, "pool_size": 16, "num_filters": 32},
-            {"filter_size": 40, "pool_size": 10, "num_filters": 64},
-        ],
-        "output_size": 16,
+        ENCODER: {
+            "type": "stacked_cnn",
+            "should_embed": False,
+            "conv_layers": [
+                {"filter_size": 400, "pool_size": 16, "num_filters": 32},
+                {"filter_size": 40, "pool_size": 10, "num_filters": 64},
+            ],
+            "output_size": 16,
+        },
         "destination_folder": folder,
     }
-    feature.update(kwargs)
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
 def timeseries_feature(**kwargs):
-    feature = {"name": "timeseries_" + random_string(), "type": "timeseries", "max_len": 7}
-    feature.update(kwargs)
+    feature = {
+        "name": "timeseries_" + random_string(),
+        "type": "timeseries",
+        ENCODER: {"type": "parallel_cnn", "max_len": 7},
+    }
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
 def binary_feature(**kwargs):
-    feature = {"name": "binary_" + random_string(), "type": "binary"}
-    feature.update(kwargs)
+    feature = {
+        "name": "binary_" + random_string(),
+        "type": "binary",
+    }
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
 def bag_feature(**kwargs):
-    feature = {"name": "bag_" + random_string(), "type": "bag", "max_len": 5, "vocab_size": 10, "embedding_size": 5}
-    feature.update(kwargs)
+    feature = {
+        "name": "bag_" + random_string(),
+        "type": "bag",
+        ENCODER: {"type": "embed", "max_len": 5, "vocab_size": 10, "embedding_size": 5},
+    }
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
@@ -349,7 +397,7 @@ def date_feature(**kwargs):
         "type": "date",
         "preprocessing": {"datetime_format": random.choice(list(DATETIME_FORMATS.keys()))},
     }
-    feature.update(kwargs)
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
@@ -357,15 +405,21 @@ def date_feature(**kwargs):
 
 def h3_feature(**kwargs):
     feature = {"name": "h3_" + random_string(), "type": "h3"}
-    feature.update(kwargs)
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
 
 
 def vector_feature(**kwargs):
-    feature = {"type": VECTOR, "vector_size": 5, "name": "vector_" + random_string()}
-    feature.update(kwargs)
+    feature = {
+        "type": VECTOR,
+        "name": "vector_" + random_string(),
+        "preprocessing": {
+            "vector_size": 5,
+        },
+    }
+    recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
     return feature
@@ -428,8 +482,8 @@ def generate_output_features_with_dependencies(main_feature, dependencies):
     """
 
     output_features = [
-        category_feature(vocab_size=2, reduce_input="sum"),
-        sequence_feature(vocab_size=10, max_len=5),
+        category_feature(decoder={"type": "classifier", "vocab_size": 2}, reduce_input="sum"),
+        sequence_feature(decoder={"type": "generator", "vocab_size": 10, "max_len": 5}),
         number_feature(),
     ]
 
@@ -454,12 +508,14 @@ def generate_output_features_with_dependencies(main_feature, dependencies):
 def generate_output_features_with_dependencies_complex():
     """Generates multiple output features specifications with dependencies."""
 
-    tf = text_feature(vocab_size=4, max_len=5, decoder="generator")
-    sf = sequence_feature(vocab_size=4, max_len=5, decoder="generator", dependencies=[tf["name"]])
+    tf = text_feature(decoder={"vocab_size": 4, "max_len": 5, "type": "generator"})
+    sf = sequence_feature(decoder={"vocab_size": 4, "max_len": 5, "type": "generator"}, dependencies=[tf["name"]])
     nf = number_feature(dependencies=[tf["name"]])
     vf = vector_feature(dependencies=[sf["name"], nf["name"]])
-    set_f = set_feature(vocab_size=4, dependencies=[tf["name"], vf["name"]])
-    cf = category_feature(vocab_size=4, dependencies=[sf["name"], nf["name"], set_f["name"]])
+    set_f = set_feature(decoder={"type": "classifier", "vocab_size": 4}, dependencies=[tf["name"], vf["name"]])
+    cf = category_feature(
+        decoder={"type": "classifier", "vocab_size": 4}, dependencies=[sf["name"], nf["name"], set_f["name"]]
+    )
 
     # The correct order ids[tf, sf, nf, vf, set_f, cf]
     # # shuffling it to test the robustness of the topological sort
@@ -501,24 +557,43 @@ def get_weights(model: torch.nn.Module) -> List[torch.Tensor]:
     return [param.data for param in model.parameters()]
 
 
+def has_no_grad(
+    val: Union[np.ndarray, torch.Tensor, str, list],
+):
+    """Checks if two values are close to each other."""
+    if isinstance(val, list):
+        return all(has_no_grad(v) for v in val)
+    if isinstance(val, torch.Tensor):
+        return not val.requires_grad
+    return True
+
+
 def is_all_close(
     val1: Union[np.ndarray, torch.Tensor, str, list],
     val2: Union[np.ndarray, torch.Tensor, str, list],
-    tolerance=1e-8,
+    tolerance=1e-4,
 ):
     """Checks if two values are close to each other."""
     if isinstance(val1, list):
         return all(is_all_close(v1, v2, tolerance) for v1, v2 in zip(val1, val2))
-
     if isinstance(val1, str):
         return val1 == val2
     if isinstance(val1, torch.Tensor):
-        val1 = val1.detach().numpy()
+        val1 = val1.cpu().detach().numpy()
     if isinstance(val2, torch.Tensor):
-        val2 = val2.detach().numpy()
+        val2 = val2.cpu().detach().numpy()
     return val1.shape == val2.shape and np.allclose(val1, val2, atol=tolerance)
 
 
+def is_all_tensors_cuda(val: Union[np.ndarray, torch.Tensor, str, list]) -> bool:
+    if isinstance(val, list):
+        return all(is_all_tensors_cuda(v) for v in val)
+
+    if isinstance(val, torch.Tensor):
+        return val.is_cuda
+    return True
+
+
 def run_api_experiment(input_features, output_features, data_csv):
     """Helper method to avoid code repetition in running an experiment.
 
@@ -571,6 +646,9 @@ def run_api_experiment(input_features, output_features, data_csv):
 
 def add_nans_to_df_in_place(df: pd.DataFrame, nan_percent: float):
     """Adds nans to a pandas dataframe in-place."""
+    if nan_percent == 0:
+        # No-op if nan_percent is 0
+        return None
     if nan_percent < 0 or nan_percent > 1:
         raise ValueError("nan_percent must be between 0 and 1")
 
@@ -586,8 +664,7 @@ def add_nans_to_df_in_place(df: pd.DataFrame, nan_percent: float):
 def read_csv_with_nan(path, nan_percent=0.0):
     """Converts `nan_percent` of samples in each row of the CSV at `path` to NaNs."""
     df = pd.read_csv(path)
-    if nan_percent > 0:
-        add_nans_to_df_in_place(df, nan_percent)
+    add_nans_to_df_in_place(df, nan_percent)
     return df
 
 
@@ -663,12 +740,49 @@ def create_data_set_to_use(data_format, raw_data, nan_percent=0.0):
         dataset_to_use = replace_file_extension(raw_data, "tsv")
         read_csv_with_nan(raw_data, nan_percent=nan_percent).to_csv(dataset_to_use, sep="\t", index=False)
 
+    elif data_format == "pandas+numpy_images":
+        df = read_csv_with_nan(raw_data, nan_percent=nan_percent)
+        processed_df_rows = []
+        for _, row in df.iterrows():
+            processed_df_row = {}
+            for feature_name, raw_feature in row.iteritems():
+                if "image" in feature_name and not (type(raw_feature) == float and np.isnan(raw_feature)):
+                    feature = np.array(Image.open(raw_feature))
+                else:
+                    feature = raw_feature
+                processed_df_row[feature_name] = feature
+            processed_df_rows.append(processed_df_row)
+        dataset_to_use = pd.DataFrame(processed_df_rows)
+
     else:
         ValueError(f"'{data_format}' is an unrecognized data format")
 
     return dataset_to_use
 
 
+def augment_dataset_with_none(
+    df: pd.DataFrame, first_row_none: bool = False, last_row_none: bool = False, nan_cols: List = []
+) -> pd.DataFrame:
+    """Optionally sets the first and last rows of nan_cols of the given dataframe to nan.
+
+    :param df: dataframe containg input features/output features
+    :type df: pd.DataFrame
+    :param first_row_none: indicates whether to set the first rowin the dataframe to np.nan
+    :type first_row_none: bool
+    :param last_row_none: indicates whether to set the last row in the dataframe to np.nan
+    :type last_row_none: bool
+    :param nan_cols: a list of columns in the dataframe to explicitly set the first or last rows to np.nan
+    :type nan_cols: list
+    """
+    if first_row_none:
+        for col in nan_cols:
+            df.iloc[0, df.columns.get_loc(col)] = np.nan
+    if last_row_none:
+        for col in nan_cols:
+            df.iloc[-1, df.columns.get_loc(col)] = np.nan
+    return df
+
+
 def train_with_backend(
     backend,
     config,
